// CATALOG - written by Dimitar Kovachev
#include <stdio.h>
#include <string.h>

struct Item
{
	int lex, ord;
	char* name;
	int name_len;
	Item *prev_ord, *next_ord;
	Item *prev_lex, *next_lex;
	Item *left, *right, *father;
};

class Catalog
{
private:
	int count;
	int sorted_lex, sorted_ord;
	Item head, tail;
	Item *root;
	Item *last_left, *last_right;

	void Add (const char*, Item*&, int);
	Item* FindStr (const char*, Item*) const;
	Item* FindNum (int) const;
	Item* FindLex (int, Item*) const;
	void DDelete (Item*);
	void DelTreeNode (Item*);

	inline void ADD (const char*, int);
	void FS (const char*);
	void FN (int) const;
	void FL (int);
	void DS (const char*);
	void DN (int);
	void DL (int);
public:
	Catalog (void);
	~Catalog (void);
	void Execute (char*);
};

FILE* fin = fopen ("catalog.inp", "rt");
FILE* fout = fopen ("catalog.out", "wt");

inline int CheckStr (const char* s, int len)
{
	return (s[0] != '\0' && s[0] != ' ' && s[len-1] != ' ');
}
inline int CheckNum (const char* s, int len)
{
	int value, count;
	if (s[0] == '\0' || s[0] == ' ') return -1;
	sscanf (s, "%d%n", &value, &count);
	if (len != count) return -1;
	else return value;
}

Catalog::Catalog (void)
{
	sorted_lex = sorted_ord = 1;
	count = 0;
	root = last_left = last_right = NULL;

	head.lex = head.ord = tail.lex = tail.ord = -1;
	head.name = tail.name = NULL;
	head.name_len = tail.name_len = 0;
	head.prev_ord = head.prev_lex = NULL;
	head.next_ord = head.next_lex = &tail;
	head.left = head.right = head.father = NULL;

	tail.next_ord = tail.next_lex = NULL;
	tail.prev_ord = tail.prev_lex = &head;
	tail.left = tail.right = tail.father = NULL;
}

Catalog::~Catalog (void)
{
	Item *del, *nxt = head.next_ord;
	while (nxt != &tail)
	{
		del = nxt;
		nxt = nxt->next_ord;
		delete [] del->name;
		delete del;
	}
}

void Catalog::Add (const char* str, Item*& where, int len)
{
	if (where == NULL)
	{
		count++;
		sorted_lex = sorted_ord = 0;
		if (last_right == NULL) last_right = &head;
		if (last_left == NULL) last_left = &tail;

		where = new Item;

		where->name_len = len;
		where->name = new char[where->name_len+1];
		strcpy (where->name, str);

		where->lex = where->ord = 0;
		where->left = where->right = where->father = NULL;

		where->prev_lex = last_right;
		last_right->next_lex = where;
		where->next_lex = last_left;
		last_left->prev_lex = where;
	
		where->next_ord = &tail;
		where->prev_ord = tail.prev_ord;
		tail.prev_ord->next_ord = where;
		tail.prev_ord = where;

		fprintf(fout, "ADDED: %s\n", str);
		return;
	}
	int i = strcmp (where->name, str);
	if (i == 0)
	{
		fprintf (fout, "DUPLICATES NOT ALLOWED\n");
		return;
	}
	if (i > 0)
	{
		last_left = where;
		Add (str, where->left, len);
		where->left->father = where;
	}
	else 
	{
		last_right = where;
		Add (str, where->right, len);
		where->right->father = where;
	}
}

inline void Catalog::ADD (const char* str, int len)
{
	last_left = last_right = NULL;
	Add (str, root, len);
}

Item* Catalog::FindStr (const char* str, Item* where) const
{
	if (where == NULL) return NULL;
	int i = strcmp (where->name, str);
	if (i == 0) return where;
	if (i > 0) return FindStr (str, where->left);
	else return FindStr (str, where->right);
}

Item* Catalog::FindNum (int n) const
{
	Item* tmp;
	int i;
	if (n < count / 2)
	{
		tmp = head.next_ord;
		for (i = 1; i < n; i++)
			tmp = tmp->next_ord;
	}
	else
	{
		tmp = tail.prev_ord;
		for (i = count; i > n; i--)
			tmp = tmp->prev_ord;
	}
	return tmp;
}
Item* Catalog::FindLex (int n, Item* where) const
{
	if (where->lex == n) return where;
	if (where->lex > n) return FindLex (n, where->left);
	else return FindLex (n, where->right);
}

void Catalog::FS (const char* str)
{
	Item *p = FindStr (str, root);
	if (p == NULL) fprintf(fout, "NOT FOUND\n");
	else 
	{
		if (sorted_ord == 0)
		{
			sorted_ord = 1;
			Item *tmp = head.next_ord;
			for (int i = 1; tmp != &tail; i++)
			{
				tmp->ord = i;
				tmp = tmp->next_ord;
			}
		}
		fprintf(fout, "FOUND: %d\n", p->ord);
	}
}

void Catalog::FN (int x) const
{
	if (x <= 0 || x > count)
	{
		fprintf(fout, "NOT FOUND\n");
		return;
	}
	fprintf(fout, "FOUND: %s\n", FindNum (x)->name);
}

void Catalog::FL (int x)
{
	if (x <= 0 || x > count)
	{
		fprintf(fout, "NOT FOUND\n");
		return;
	}

	Item* p;
	if (sorted_lex == 0)
	{
		sorted_lex = 1;
		Item* tmp = head.next_lex;
		for (int i = 1; i < x; i++)
		{
			tmp->lex = i;
			tmp = tmp->next_lex;
		}
		p = tmp;
		while (tmp != &tail)
		{
			tmp->lex = i++;
			tmp = tmp->next_lex;
		}
	}
	else p = FindLex (x, root);
	fprintf(fout, "FOUND: %s\n", FindLex (x, root)->name);
}

void Catalog::DS (const char* str)
{
	Item *p = FindStr (str, root);
	if (p == NULL) fprintf(fout, "NOT FOUND\n");
	else 
	{
		if (sorted_ord == 0)
		{
			sorted_ord = 1;
			Item *tmp = head.next_ord;
			for (int i = 1; tmp != &tail; i++)
			{
				tmp->ord = i;
				tmp = tmp->next_ord;
			}
		}
		fprintf(fout, "DELETED: %d\n", p->ord);
		DDelete (p);
	}
}

void Catalog::DN (int x)
{
	if (x <= 0 || x > count)
	{
		fprintf(fout, "NOT FOUND\n");
		return;
	}
	Item *p = FindNum (x);
	fprintf(fout, "DELETED: %s\n", p->name);
	DDelete (p);
}
void Catalog::DL (int x)
{
	int i;
	if (x <= 0 || x > count)
	{
		fprintf(fout, "NOT FOUND\n");
		return;
	}

	if (sorted_lex == 0)
	{
		Item* tmp = head.next_lex;
		for (i = 1; i < x; i++)
		{
			tmp->lex = i;
			tmp = tmp->next_lex;
		}
		Item *p = tmp;
		tmp = tmp->next_lex;
		for (; tmp != &tail; i++)
		{
			tmp->lex = i;
			tmp = tmp->next_lex;
		}
		fprintf(fout, "DELETED: %s\n", p->name);
		DDelete (p);
		sorted_lex = 1;
	}
	else
	{
		Item *p = FindLex (x, root);
		fprintf(fout, "DELETED: %s\n", p->name);
		DDelete (p);
	}
}
void Catalog::DDelete (Item* p)
{
	sorted_lex = sorted_ord = 0;
	count--;

	p->prev_ord->next_ord = p->next_ord;
	p->next_ord->prev_ord = p->prev_ord;

	p->prev_lex->next_lex = p->next_lex;
	p->next_lex->prev_lex = p->prev_lex;

	DelTreeNode (p);
}

void Catalog::DelTreeNode (Item* p)
{
	if (p->left != NULL && p->right != NULL)
	{
		Item* replace = p->right;
		while (replace->left != NULL)
			replace = replace->left;

		p->lex = replace->lex;
		p->ord = replace->ord;

		delete [] p->name;
		p->name = replace->name;
		replace->name = NULL;
		p->name_len = replace->name_len;

		p->next_ord = replace->next_ord;
		p->prev_ord = replace->prev_ord;
		p->next_lex = replace->next_lex;
		p->prev_lex = replace->prev_lex;
		replace->next_ord->prev_ord = p;
		replace->prev_ord->next_ord = p;
		replace->next_lex->prev_lex = p;
		replace->prev_lex->next_lex = p;

		DelTreeNode (replace);
    }
	else
	{
		Item* replace;
		if (p->left == NULL) replace = p->right;
		else replace = p->left;

		if (replace != NULL)
			replace->father = p->father;

		if (p == root) root = replace;
		else
		{
			if (p == p->father->left)
				p->father->left = replace;
			else p->father->right = replace;
		}
		delete [] p->name;
		delete p;
    }
}

void Catalog::Execute (char* buf)
{
	int i, str_len;
	str_len = strlen (buf) - 1;
	buf[str_len] = '\0';
	if (strncmp (buf, "ADD ", 4) == 0)
		if (CheckStr (buf+4, str_len - 4) == 1)
		{
			ADD (buf+4, str_len - 4);
			return;
		}
	if (buf[2] != ' ')
	{
		fprintf(fout, "SYNTAX ERROR\n");
		return;
	}

	if (buf[0] == 'F')
	{
		if (buf[1] == 'S')
			if (CheckStr (buf+3, str_len - 3) == 1)
			{
				FS (buf+3);
				return;
			}
		if (buf[1] == 'N')
			if ((i = CheckNum (buf+3, str_len - 3)) != -1)
			{
				FN (i);
				return;
			}
		if (buf[1] == 'L')
			if ((i = CheckNum (buf+3, str_len - 3)) != -1)
			{
				FL (i);
				return;
			}
	}
	if (buf[0] == 'D')
	{
		if (buf[1] == 'S')
			if (CheckStr (buf+3, str_len - 3) == 1)
			{
				DS (buf+3);
				return;
			}
		if (buf[1] == 'N')
			if ((i = CheckNum (buf+3, str_len - 3)) != -1)
			{
				DN (i);
				return;
			}
		if (buf[1] == 'L')
			if ((i = CheckNum (buf+3, str_len - 3)) != -1)
			{
				DL (i);
				return;
			}
	}
	fprintf(fout, "SYNTAX ERROR\n");
}

int main (void)
{
	Catalog C;
	char bufr[260];

	while (fgets(bufr, 259, fin) != NULL)
		C.Execute (bufr);

	fclose (fin);
	fclose (fout);

	return 0;
}