//  1
// m_roussev@yahoo.com
//    / 29 . / .  /  


#include <vector>
#include <fstream>
#include <iostream>
#include <string>
#include <sstream>

using namespace std;

const int PROGRAM_ERROR_NO_ERROR = 0;
const int PROGRAM_ERROR_CANNOT_OPEN_INPUT_FILE = 1;
const int PROGRAM_ERROR_CANNOT_CREATE_OUTPUT_FILE = 2;


const string FILE_NAME_INPUT = "CATALOG.INP";
const string FILE_NAME_OUTPUT = "CATALOG.OUT";
const int COMMAND_LINE_LENGTH_MAX = 250;


const int MAX_NUMBER_OF_LRU_ITEMS = 1024;

const int ITEM_NAME_LENGTH_MAX = 250;
const int ITEM_NAME_STEP = 8;
const int TEMP_FILE_COUNT = (ITEM_NAME_LENGTH_MAX / ITEM_NAME_STEP) + 1;


const string TEMP_FILE_PRE = "~f";
const string TEMP_FILE_EXT = ".tmp";


const string ARG_DIV = " ";
const string ITEM_NAME_MAX_VALUE = "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
const string ITEM_NAME_MIN_VALUE = " ";


const string COMMAND_ADD = "ADD" ;
const string COMMAND_FS  = "FS"  ;
const string COMMAND_FN  = "FN"  ;
const string COMMAND_FL  = "FL"  ;
const string COMMAND_DS  = "DS"  ;
const string COMMAND_DN  = "DN"  ;
const string COMMAND_DL  = "DL"  ;

const string RESULT_SYNTAX_ERROR = "SYNTAX ERROR" ;
const string RESULT_ADDED = "ADDED: " ;
const string RESULT_DUPLICATES_NOT_ALLOWED = "DUPLICATES NOT ALLOWED";
const string RESULT_FOUND = "FOUND: " ;
const string RESULT_NOT_FOUND = "NOT FOUND" ;
const string RESULT_DELETED = "DELETED: ";




string items_name_min = ITEM_NAME_MAX_VALUE ;
string items_name_max = ITEM_NAME_MIN_VALUE ;
ofstream data_output;


int StrToInt( const string& s)
{
  istringstream instr(s);
  int n;
  instr >> n;
  return n;
} // int StrToInt( string s)

string IntToStr( const int n)
{
  ostringstream outstr;
  outstr << n;
  return outstr.str();
} // string IntToStr( const int n)


bool IsNumber( const string& s)
{
  return (s.find_first_not_of("0123456789") == s.npos);
} // bool IsNumber( string s)


class DiskStorage
{
public:
  void Open();
  void Close(); 
  string ReadItem(const int aType, const int aPosition);
  void WriteItem(const int aType, const int aPosition, const string& aValue);
private:
  fstream temp_file[TEMP_FILE_COUNT];
  string temp_file_names [TEMP_FILE_COUNT];
  
}; //class DiskStorage

void DiskStorage::Open() 
{
  //       

  for (int i=0; i<TEMP_FILE_COUNT; i++)
  {
    temp_file_names [i] = TEMP_FILE_PRE + IntToStr(i) + TEMP_FILE_EXT;
    temp_file[i].open ( temp_file_names [i].c_str(), ios::in | ios::out | ios::trunc | ios::binary);
  }; // for
  
} // void DiskStorage::Open()

void DiskStorage::Close()
{
  //      
  for ( int i=0; i<TEMP_FILE_COUNT; i++ )
  {
    temp_file[i].close();
    remove (temp_file_names[i].c_str());
  }; // for
  
} // void DiskStorage::Close()

string DiskStorage::ReadItem(const int aType, const int aPosition)
{
  string aItemName;

  temp_file[aType].seekg( aPosition*(aType+1)*ITEM_NAME_STEP, ios::beg) ;

  getline (temp_file[aType], aItemName);
  
  return aItemName;
} // string DiskStorage::ReadItem(const int aType, const int aPosition)

void DiskStorage::WriteItem(const int aType, const int aPosition, const string& aValue)
{
  temp_file[aType].seekp( aPosition*(aType+1)*ITEM_NAME_STEP, ios::beg) ;
  temp_file[aType] << aValue << endl;
} // void DiskStorage::WriteItem(const int aType, const int aPosition, const string& aValue)




class ItemsIndex
{
public:
  int AddNumber (const int aType, const int aNumber);
  void FindNumber (const int aNumber, int& aType, int& aPosition);
  int GetNumber (const int aType, const int aPosition)
  {
    return MyList[aType][aPosition] ;
  };
  void SetNumber (const int aNumber, const int aType, const int aPosition)
  {
    MyList[aType][aPosition] = aNumber;
  };
  int GetPosition (const int aNumber, const int aType)
  {
    unsigned int i;
    for ( i=0; (i<MyList[aType].size()) && (MyList[aType][i] != aNumber ); i++ ) ;
    return i;
  };
  
  void DeleteNumber (const int aNumber, int& aType, int& aPosition); 
  void IncrementNumber (const int aNumber); 
private:
  vector<int> MyList[TEMP_FILE_COUNT];
  
}; //class ItemsIndex



int ItemsIndex::AddNumber (const int aType, const int aNumber)
{
  unsigned int i;
  for ( i=0; (i<MyList[aType].size()) && (MyList[aType][i] != 0); i++ ) ;
  
  if (i == MyList[aType].size())
  {
    MyList[aType].push_back (aNumber) ;
    // to do cach(bad_alloc)!!
  }
  else
  {
    MyList[aType][i] = aNumber ;  
  }
  
  return i;
  
} // int ItemsIndex::AddItem


void ItemsIndex::FindNumber (const int aNumber, int& aType, int& aPosition)
{
  int item_type;
  unsigned int i;
  bool not_found = true;
  for ( item_type=0; (item_type<TEMP_FILE_COUNT) && (not_found); item_type++)
  {
    for (i=0; i<MyList[item_type].size(); i++ )
      if (MyList[item_type][i] == aNumber)
      {
        not_found = false;
        aType = item_type;
        aPosition = i;
        break;
      } // if
      
  } // for i<TEMP_FILE_COUNT
  
} // void ItemsIndex::FindNumber 

void ItemsIndex::DeleteNumber (const int aNumber, int& aType, int& aPosition)
{
  int item_type;
  unsigned int i;
  
  for ( item_type=0; item_type<TEMP_FILE_COUNT ; item_type++)
  {
    for (i=0; i<MyList[item_type].size(); i++ )
    {
      if (MyList[item_type][i] > aNumber) 
        MyList[item_type][i]--;
      else
        if (MyList[item_type][i] == aNumber)
        {
          MyList[item_type][i] = 0;
          aType = item_type ;
          aPosition = i ;
        }
    }  // for i<MyList[item_type].size()
    
  } // for i<TEMP_FILE_COUNT
} // void ItemsIndex::DeleteNumber

void ItemsIndex::IncrementNumber (const int aNumber)
{
  int item_type;
  unsigned int i;
  
  for ( item_type=0; item_type<TEMP_FILE_COUNT ; item_type++)
  {
    for (i=0; i<MyList[item_type].size(); i++ )
    {
      if (MyList[item_type][i] >= aNumber) 
        MyList[item_type][i]++;
    }  // for i<MyList[item_type].size()
    
  } // for i<TEMP_FILE_COUNT
} // void ItemsIndex::IncrementNumber


class LRU {
public:
  LRU();
  void AddItem( const string& aItemName, const int aType, const int aInputNumber, const int aLexicalNumber) ; 
  bool FindItem( const string& aItemName, int& LexicalNumberMin, int& LexicalNumberMax, const int items_count ) ;
  bool FindItemByInputNumber( const int aInputNumber, string& aItemName ) 
  {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if (InputNumber [i] == aInputNumber) 
      {
        aItemName = ItemName [i] ;
        return true;
      }
      return false;
      
  } // FindItemByInputNumber
  
  bool FindItemByLexicalNumber( const int aLexicalNumber, string& aItemName ) 
  {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if (LexicalNumber [i] == aLexicalNumber) 
      {
        aItemName = ItemName [i] ;
        return true;
      }
      return false;
      
  } // FindItemByInputNumber
  
  
  
  void DeleteItem( int aInputNumber, int aLexicalNumber ) {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if ( (LexicalNumber [i] == aLexicalNumber) || (InputNumber [i] == aInputNumber) )
      {
        ItemName [i] = "";
        Type [i] = 0;
        InputNumber [i] = 0;
        LexicalNumber [i]= 0;
        return ;
      };
  }
  void IncrementLexAfterN( int aNumber ) {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if ( LexicalNumber [i] >= aNumber ) LexicalNumber [i]++ ;
  }
  void DecrementLexAfterN( int aNumber ) {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if ( LexicalNumber [i] > aNumber ) LexicalNumber [i]-- ;
  }
  void DecrementInputAfterN( int aNumber ) {
    for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
      if ( InputNumber [i] > aNumber ) InputNumber [i]-- ;
  }
  
private:
  int Position;
  string ItemName [MAX_NUMBER_OF_LRU_ITEMS];
  int Type [MAX_NUMBER_OF_LRU_ITEMS];
  int InputNumber [MAX_NUMBER_OF_LRU_ITEMS];
  int LexicalNumber [MAX_NUMBER_OF_LRU_ITEMS];
}; // class LRU

LRU::LRU()
{
  Position = 0 ;
  for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
  {
    ItemName [i] = "";
    Type [i] = 0 ;
    InputNumber [i] = 0;
    LexicalNumber [i]= 0;
  };
}


//   ,    
void LRU::AddItem(const string& aItemName, const int aType, const int aInputNumber, const int aLexicalNumber) 
{
  int i;
  for ( i=0; (i < MAX_NUMBER_OF_LRU_ITEMS) && (ItemName [i]!=aItemName);i++)	;
  
  if ( i == MAX_NUMBER_OF_LRU_ITEMS )	
  {
    
    ItemName [Position] = aItemName;
    Type [Position] = aType;
    InputNumber [Position] = aInputNumber;
    LexicalNumber [Position]= aLexicalNumber;
    
    Position++;
    if ( Position == MAX_NUMBER_OF_LRU_ITEMS )
      Position = 0;
    
  }
} // AddItem

bool LRU::FindItem( const string& aItemName, int& LexicalNumberMin, int& LexicalNumberMax, const int items_count )
{
  LexicalNumberMin = 1 ;
  LexicalNumberMax = items_count;
  
  
  for (int i=0; i < MAX_NUMBER_OF_LRU_ITEMS; i++)
  {
    if ( ItemName [i] == "" ) continue ;
    
    if ( aItemName != ItemName [i] )
    {
      if ( aItemName > ItemName [i] )
      {
        if ( LexicalNumberMin < LexicalNumber [i] )
          LexicalNumberMin = LexicalNumber [i] ;
      }
      else
      {
        if ( LexicalNumberMax > LexicalNumber [i] )
          LexicalNumberMax = LexicalNumber [i] ;
      }
    }
    else
    {
      LexicalNumberMin = LexicalNumber[i] ;
      LexicalNumberMax = LexicalNumber[i] ;
      return true;
    } ;
  }; // for
  
  return false;
} // FindItem



void PrintMessage(const string& message_text)
{ 
  static string last_line = "" ;
  
  if ( message_text != "" ) 
  {
    
    if ( last_line != "" )
    {
      data_output << last_line << endl ;
    }

    last_line = message_text ;

  }
  else
  {
    
    if ( last_line != "" )
    {
      data_output << last_line ;
    }
  }

} // void PrintMessage


string GetItemByLexNumber(ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, const int aLexNumber);
string GetItemByInputNumber(ItemsIndex& aInputIndex, ItemsIndex& aLexIndex, DiskStorage& aTempFile, LRU& aLRU, const int aInputNumber);


bool FindIntemByName (const string& item_name, const int LexicalNumberMin, const int LexicalNumberMax, int& MustBeLexicalNumber, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU)
{
  int LexNumMin = LexicalNumberMin;
  int LexNumMax = LexicalNumberMax;
  int LexNumAvg ;
  string item_name_help;
  
  
  while (LexNumMin <= LexNumMax)
  {
    LexNumAvg = LexNumMin + ( LexNumMax - LexNumMin) / 2;
    
    item_name_help = GetItemByLexNumber( aLexIndex, aInputIndex, aTempFile, aLRU, LexNumAvg);
    
    if (item_name_help == item_name)
    { //  
      MustBeLexicalNumber = LexNumAvg ;
      return true ;
    }  // if  
    else
    { //   
      if (item_name_help < item_name)
        LexNumMin = LexNumAvg + 1;
      else
        LexNumMax = LexNumAvg - 1;
    } //   
  }// while (LexNumMin != LexNumMax)
  
  //MustBeLexicalNumber = LexNumAvg ;
  MustBeLexicalNumber = LexNumMin ;
  return false;
} // bool FindIntemByName


bool GlobalFindItem(const string& item_name, int& MustBeLexicalNumber, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, const int items_count) 
{
  int LexicalNumberMin ;
  int LexicalNumberMax ;
  string item_name_help;	
  
  //    -  - 
  if (item_name < items_name_min ) {
    MustBeLexicalNumber = 1 ;
    return false ;
  }
  
  if (item_name > items_name_max ) {
    MustBeLexicalNumber = items_count + 1 ;
    return false ;
  }
  
  //   LRU
  if (aLRU.FindItem(item_name, LexicalNumberMin, LexicalNumberMax, items_count)) {
    MustBeLexicalNumber = LexicalNumberMin ;
    return true ;
  } // if
  else {
    
    //     
    if (FindIntemByName (item_name, LexicalNumberMin, LexicalNumberMax, MustBeLexicalNumber, aLexIndex, aInputIndex, aTempFile, aLRU))
      return true;
    else 
      return false;
    
  } // else
  
} // bool GlobalFindItem() 

void CommandADD(const string& new_item, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int& items_count)
{
  int MustBeLexicalNumber;
  int item_type;
  int item_position;
  
  if (GlobalFindItem(new_item, MustBeLexicalNumber, aLexIndex, aInputIndex, aTempFile, aLRU, items_count))
  { //  
    PrintMessage (RESULT_DUPLICATES_NOT_ALLOWED);
  } // if  
  else
  { //  ,    
    items_count++;
    if ( new_item > items_name_max )
      items_name_max = new_item ;
    
    if ( new_item < items_name_min )
      items_name_min = new_item ;
    
    item_type = new_item.size() / ITEM_NAME_STEP;
    
    aLexIndex.IncrementNumber(MustBeLexicalNumber);
    
    item_position = aLexIndex.AddNumber(item_type,MustBeLexicalNumber);
    
    aInputIndex.AddNumber(item_type,items_count);
    
    aTempFile.WriteItem(item_type, item_position, new_item);  
    
    
    aLRU.IncrementLexAfterN( MustBeLexicalNumber ); 
    aLRU.AddItem(new_item, item_type , items_count, MustBeLexicalNumber) ; //, item_position);
    
    
    PrintMessage (RESULT_ADDED+new_item);
    
  } // else   ,    
  
} // void CommandADD

void CommandFS(const string& new_item, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int& items_count)
{
  int MustBeLexicalNumber;
  int item_type;
  int item_position;
  int item_input_number;
  
  if (GlobalFindItem(new_item, MustBeLexicalNumber, aLexIndex, aInputIndex, aTempFile, aLRU, items_count))
  { //  
    
    item_type = new_item.size() / ITEM_NAME_STEP;
    item_position = aLexIndex.GetPosition( MustBeLexicalNumber, item_type) ;
    
    item_input_number = aInputIndex.GetNumber(item_type, item_position);
    
    PrintMessage ( RESULT_FOUND+IntToStr(item_input_number) );
    
  } // if  
  else
  { //  
    PrintMessage ( RESULT_NOT_FOUND );
  } // else   
  
} // void CommandFS



void CommandFN(const string& new_item, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int items_count)
{
  int item_input_number;
  
  if (IsNumber(new_item))
  { //    
    item_input_number = StrToInt( new_item );
    if ( (item_input_number < 1) || (item_input_number > items_count) )
    {
      PrintMessage ( RESULT_NOT_FOUND );
    }
    else
    {  //  
      PrintMessage ( RESULT_FOUND+GetItemByInputNumber(aInputIndex, aLexIndex, aTempFile, aLRU, item_input_number) );
    }  // else  
  } // if     
  else
  {  //     
    PrintMessage( RESULT_SYNTAX_ERROR );
  }  // else     
} // void CommandFN



void CommandFL(const string& new_item, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int items_count)
{
  int item_lexical_number;
  
  if (IsNumber(new_item))
  { //    
    item_lexical_number = StrToInt( new_item );
    if ( (item_lexical_number < 1) || (item_lexical_number > items_count) )
    {
      PrintMessage ( RESULT_NOT_FOUND );
    }
    else
    {  //  
      PrintMessage ( RESULT_FOUND+GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, item_lexical_number) );
    }  // else  
  } // if     
  else
  {  //     
    PrintMessage( RESULT_SYNTAX_ERROR );
  }  // else     
} // void CommandFL

void CommandDS(const string& new_item, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int& items_count)
{
  int item_lexical_number;
  int item_input_number;
  int item_type;
  int item_position;
  bool is_name_max = false;
  bool is_name_min = false;
  
  if ( GlobalFindItem(new_item, item_lexical_number, aLexIndex, aInputIndex, aTempFile, aLRU, items_count) == false )
  { //  
    PrintMessage ( RESULT_NOT_FOUND );
  } // if  
  else
  { //  ,    
    if ( new_item == items_name_max )
      is_name_max = true;
    
    if ( new_item == items_name_min )
      is_name_min = true;
    
    
    aLexIndex.DeleteNumber( item_lexical_number, item_type, item_position );
    
    item_input_number = aInputIndex.GetNumber( item_type, item_position );
    aInputIndex.DeleteNumber( item_input_number, item_type, item_position);
    
    aLRU.DeleteItem( item_input_number, item_lexical_number );
    aLRU.DecrementInputAfterN(item_input_number);
    aLRU.DecrementLexAfterN(item_lexical_number);
    
    items_count--;
    
    if ( items_count == 0 )
    {
      items_name_min = ITEM_NAME_MAX_VALUE ;
      items_name_max = ITEM_NAME_MIN_VALUE ;
    }
    else
    {
      if ( is_name_max )
        items_name_max = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, items_count) ;
      
      if ( is_name_min )
        items_name_min = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, 1) ;
    }
    
    
    PrintMessage (RESULT_DELETED+IntToStr(item_input_number));    
    
  } // else  ,    
  
} // void CommandDS

void CommandDN(const string& new_item_number, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int& items_count)
{
  int item_input_number;
  int item_lexical_number;
  int item_type;
  int item_position;
  string item_name;
  bool is_name_max = false;
  bool is_name_min = false;
  
  
  if (IsNumber(new_item_number))
  { //    
    item_input_number = StrToInt( new_item_number );
    if ( (item_input_number < 1) || (item_input_number > items_count) )
    {
      PrintMessage ( RESULT_NOT_FOUND );
    }
    else
    {  //    
      
      item_name = GetItemByInputNumber(aInputIndex, aLexIndex, aTempFile, aLRU, item_input_number) ;
      
      if ( item_name == items_name_max )
        is_name_max = true;
      
      if ( item_name == items_name_min )
        is_name_min = true;
      
      aInputIndex.DeleteNumber( item_input_number, item_type, item_position );
      
      item_lexical_number = aLexIndex.GetNumber( item_type, item_position );
      
      aLexIndex.DeleteNumber( item_lexical_number, item_type, item_position);
      
      aLRU.DeleteItem( item_input_number, item_lexical_number );
      aLRU.DecrementInputAfterN(item_input_number);
      aLRU.DecrementLexAfterN(item_lexical_number);
      
      items_count--;
      
      if ( items_count == 0 )
      {
        items_name_min = ITEM_NAME_MAX_VALUE ;
        items_name_max = ITEM_NAME_MIN_VALUE ;
      }
      else
      {
        if ( is_name_max )
          items_name_max = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, items_count) ;
        
        if ( is_name_min )
          items_name_min = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, 1) ;
      }
      
      PrintMessage ( RESULT_DELETED+item_name );
      
    }  // else    
    
    
  } // if     
  else
  {  //     
    PrintMessage( RESULT_SYNTAX_ERROR );
  }  // else     
} // void CommandDN


void CommandDL(const string& new_item_number, ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, int& items_count)
{
  int item_input_number;
  int item_lexical_number;
  int item_type;
  int item_position;
  string item_name;
  bool is_name_max = false;
  bool is_name_min = false;
  
  
  if (IsNumber(new_item_number))
  { //    
    item_lexical_number = StrToInt( new_item_number );
    if ( (item_lexical_number < 1) || (item_lexical_number > items_count) )
    {
      PrintMessage ( RESULT_NOT_FOUND );
    }
    else
    {  //    
      
      if ( aLRU.FindItemByLexicalNumber(item_lexical_number, item_name) == false )
        item_name = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, item_lexical_number) ;
      
      if ( item_name == items_name_max )
        is_name_max = true;
      
      if ( item_name == items_name_min )
        is_name_min = true;
      
      
      aLexIndex.DeleteNumber( item_lexical_number, item_type, item_position );
      item_input_number = aInputIndex.GetNumber( item_type, item_position );
      aInputIndex.DeleteNumber( item_input_number, item_type, item_position);
      
      
      aLRU.DeleteItem( item_input_number, item_lexical_number );
      aLRU.DecrementInputAfterN(item_input_number);
      aLRU.DecrementLexAfterN(item_lexical_number);
      
      items_count--;
      
      if ( items_count == 0 )
      {
        items_name_min = ITEM_NAME_MAX_VALUE ;
        items_name_max = ITEM_NAME_MIN_VALUE ;
      }
      else
      {
        if ( is_name_max )
          items_name_max = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, items_count) ;
        
        if ( is_name_min )
          items_name_min = GetItemByLexNumber(aLexIndex, aInputIndex, aTempFile, aLRU, 1) ;
      }
      
      PrintMessage ( RESULT_DELETED+item_name );
      
    }  // else    
    
    
  } // if     
  else
  {  //     
    PrintMessage( RESULT_SYNTAX_ERROR );
  }  // else     
} // void CommandDL




//---------------------------------------------------------------------------
int main()
{
  int items_count = 0;
  DiskStorage temp_file;
  ifstream data_input;
  ItemsIndex LexicalNumbers;
  ItemsIndex InputNumbers;
  LRU LRUItems;
  
  
  
  data_input.open (FILE_NAME_INPUT.c_str(), ios::in);
  if ( data_input.fail() )
  {
    cerr << "PROGRAM_ERROR_CANNOT_OPEN_INPUT_FILE " << FILE_NAME_INPUT << endl;
    
    return PROGRAM_ERROR_CANNOT_OPEN_INPUT_FILE ;
  };
  
  data_output.open (FILE_NAME_OUTPUT.c_str(), ios::trunc | ios::out );
  if ( data_output.fail() ) 
  {
    data_input.close();
    cerr << "PROGRAM_ERROR_CANNOT_CREATE_OUTPUT_FILE " << FILE_NAME_OUTPUT << endl;
    
    return PROGRAM_ERROR_CANNOT_CREATE_OUTPUT_FILE ;
  };
  
  
  temp_file.Open();
  
 
  
  string items ;
  string command ;
  string argument;
  size_t p;
  
  
  //  ,    .
  char x;
  data_input.get(x);
  data_input.putback(x);



  while ( !data_input.eof()) 
  {
    
    getline (data_input, items) ;
    
    if (items.size() > COMMAND_LINE_LENGTH_MAX)
    { 
      PrintMessage( RESULT_SYNTAX_ERROR );
      continue ;
    }
    
    p = items.find ( ARG_DIV ) ;
    if ( p != items.npos ) 
    {
      
      //      
      command.assign (items,0,p) ;
      argument.assign (items,p + ARG_DIV.size(), items.size()-p - ARG_DIV.size()) ;
      
     
      //     
      if ( (argument < ITEM_NAME_MIN_VALUE) || (argument > ITEM_NAME_MAX_VALUE ) || (argument.rfind (" ")+1 == argument.size()) ) 
      {
        PrintMessage( RESULT_SYNTAX_ERROR );
        continue ;
      }  ;
      
      
      if  		( command == COMMAND_ADD ) CommandADD( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_FS  ) CommandFS( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_FN  ) CommandFN( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_FL  ) CommandFL( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_DS  ) CommandDS( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_DN  ) CommandDN( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else if ( command == COMMAND_DL  ) CommandDL( argument, LexicalNumbers, InputNumbers, temp_file, LRUItems, items_count ) ;
      else PrintMessage( RESULT_SYNTAX_ERROR );
      
    }
    else
    {
      PrintMessage( RESULT_SYNTAX_ERROR );
    }
  }; // while EOF
  
  
  //   
  PrintMessage("");  
  
  temp_file.Close();
  data_input.close();
  data_output.close ();
  
  return PROGRAM_ERROR_NO_ERROR;
}


string GetItemByLexNumber(ItemsIndex& aLexIndex, ItemsIndex& aInputIndex, DiskStorage& aTempFile, LRU& aLRU, const int aLexNumber)
{
  int item_type;
  int file_position ;
  string item_name;
  
  aLexIndex.FindNumber(aLexNumber, item_type, file_position);
  item_name = aTempFile.ReadItem(item_type, file_position);
  aLRU.AddItem( item_name, item_type, aInputIndex.GetNumber(item_type, file_position), aLexNumber) ;
  return item_name;
} // string GetItemByNumber

string GetItemByInputNumber(ItemsIndex& aInputIndex, ItemsIndex& aLexIndex, DiskStorage& aTempFile, LRU& aLRU, const int aInputNumber)
{
  int item_type;
  int file_position ;
  string item_name;
  
  
  if ( aLRU.FindItemByInputNumber(aInputNumber, item_name ) == false ) 
  {
    aInputIndex.FindNumber(aInputNumber, item_type, file_position);
    item_name = aTempFile.ReadItem(item_type, file_position);
    aLRU.AddItem( item_name, item_type, aInputNumber, aLexIndex.GetNumber(item_type, file_position)) ; 
  }
  return item_name;
} // string GetItemByNumber
