#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>

#define LINE_LEN 300

typedef struct { int i_n, l_n; char desc[LINE_LEN]; } item;

using namespace std;

vector<item *> data;
vector<int> inp, lex;

char read_buffer[LINE_LEN];
char result[LINE_LEN];
char command[LINE_LEN];
char * params;

void add_item();

void find_item_lex();
void find_item_inp();
void find_item_inp_lex();

void delete_item_lex();
void delete_item_inp();
void delete_item_inp_lex();

void print_lex()
{
	int i;

	printf( "\nDATA\n" );
	for ( i = 0; i < data.size(); i++ )
	{
		printf( "%d %d %s\n", data[i]->i_n, data[i]->l_n, data[i]->desc );
	}

	printf( "\nINP\n" );
	for ( i = 0; i < inp.size(); i++ )
	{
		printf( "%d %d %s\n", data[inp[i]]->i_n, data[inp[i]]->l_n, data[inp[i]]->desc );
	}

	printf( "\nLEX\n" );
	for ( i = 0; i < lex.size(); i++ )
	{
		printf( "%d %d %s\n", data[lex[i]]->i_n, data[lex[i]]->l_n, data[lex[i]]->desc );
	}
	
	printf( "-----------------------------------\n" );

	char temp[200];
	scanf( "%s", temp );
}

void process_command( void )
{
	int p;

	result[0] = '\0';

	params = NULL;
	command[0] = '\0';

	for ( p = 0; read_buffer[p]; p++ )
	{
		if ( read_buffer[p] == ' ' )
		{
			memcpy( command, read_buffer, sizeof(char)*p );
			command[p] = '\0';
			params = read_buffer + p + 1;
			break;
		}
	}

	while ( read_buffer[p] ) p++;

	while ( p && ( ( read_buffer[p - 1] == '\n' ) ||
				   ( read_buffer[p - 1] == '\r' ) ||
				   ( read_buffer[p - 1] == ' ' ) ||
				   ( read_buffer[p - 1] == '\t' ) ) )
	{
		read_buffer[--p] = '\0';
	}

	if ( params == NULL )
	{
		strcpy( result, "SYNTAX ERROR" );
	}
	else
	if ( !strcmp( command, "ADD" ) )
	{
		add_item();
	}
	else
	if ( !strcmp( command, "FS" ) )
	{
		find_item_lex();
	}
	else
	if ( !strcmp( command, "FN" ) )
	{
		find_item_inp();
	}
	else
	if ( !strcmp( command, "FL" ) )
	{
		find_item_inp_lex();
	}
	else
	if ( !strcmp( command, "DS" ) )
	{
		delete_item_lex();
	}
	else
	if ( !strcmp( command, "DN" ) )
	{
		delete_item_inp();
	}
	else
	if ( !strcmp( command, "DL" ) )
	{
		delete_item_inp_lex();
	}
	else
	{
		strcpy( result, "SYNTAX ERROR" );
	}
}

int main( void )
{
	int first_output = 1;
	FILE * f_in, * f_out;

	f_in = fopen( "catalog.inp", "r" );
	f_out = fopen( "catalog.out", "w" );

	while ( fgets( read_buffer, LINE_LEN, f_in ) )
	{
		process_command();
		if ( first_output )
		{
			fprintf( f_out, "%s", result );
			first_output = 0;
		}
		else
		{
			fprintf( f_out, "\n%s", result );
		}

//		print_lex();
//		fflush( f_out );
	}

	fclose( f_in );
	fclose( f_out );

	for ( vector<item *>::iterator i = data.begin(); i != data.end(); i++ )
	{
		free( (*i) );
	}
	data.clear();

	return 0;
}

int find_lex( void )
{
	int l = 0, r = lex.size() - 1, p, t;
	char * x;

	while ( l <= r )
	{
		x = data[ lex[ p = (l + r) / 2 ] ]->desc;

		t = strcmp( params, x );

		if ( t == 0 ) return lex[p]; else
		if ( t > 0 ) l = p + 1; else
		if ( t < 0 ) r = p - 1;
	}

	return -1;
}

int insert_lex( void )
{
	int l = 0, r = lex.size() - 1, p, t;
	char * x;

	while ( l <= r )
	{
		x = data[ lex[ p = (l + r) / 2 ] ]->desc;

		t = strcmp( params, x );

		if ( t == 0 ) return -1; else
		if ( t > 0 ) l = p + 1; else
		if ( t < 0 ) r = p - 1;
	}

	return l;
}

void add_item( void )
{
	int p, k;
	item * new_item;

	p = insert_lex();

	if ( p == -1 )
	{
		strcpy( result, "DUPLICATES NOT ALLOWED" );
		return;
	}

	new_item = (item *) malloc( sizeof(item) );
	strcpy( new_item->desc, params );

	k = data.size();
	new_item->i_n = k;
	new_item->l_n = p;
	data.push_back( new_item );

	if ( lex.size() == 0 )
	{
		lex.push_back(k);
	}
	else
	{
		lex.insert( lex.begin() + p, k );

		for ( vector<int>::iterator iter = lex.begin() + p + 1; iter != lex.end(); iter++ )
		{
			data[ (*iter) ]->l_n++;
		}
	}

	inp.push_back(k);

	sprintf( result, "ADDED: %s", params );
}

void find_item_lex( void )
{
	int p = find_lex();

	if ( p == -1 )
	{
		strcpy( result, "NOT FOUND" );
	}
	else
	{
		sprintf( result, "FOUND: %d", data[p]->i_n + 1 );
	}
}

void find_item_inp()
{
	int k = atoi( params ) - 1;

	if ( ( k >= 0 ) && ( k < inp.size() ) )
	{
		sprintf( result, "FOUND: %s", data[ inp[k] ]->desc );
	}
	else
	{
		strcpy( result, "NOT FOUND" );
	}
}

void find_item_inp_lex()
{
	int k = atoi( params ) - 1;

	if ( ( k >= 0 ) && ( k < lex.size() ) )
	{
		sprintf( result, "FOUND: %s", data[ lex[k] ]->desc );
	}
	else
	{
		strcpy( result, "NOT FOUND" );
	}
}

void real_delete_item( int p )
{
	int lex_no, inp_no, n;
	vector<int>::iterator iter;

	inp_no = data[p]->i_n;
	lex_no = data[p]->l_n;

	inp.erase( inp.begin() + inp_no );
	lex.erase( lex.begin() + lex_no );

	for ( iter = inp.begin() + inp_no; iter != inp.end(); iter++ ) data[ (*iter) ]->i_n--;
	for ( iter = lex.begin() + lex_no; iter != lex.end(); iter++ ) data[ (*iter) ]->l_n--;

	free( data[p] );
	n = data.size();

	if ( p != n - 1 )
	{
		data[p] = data[n - 1];
		inp[ data[p]->i_n ] = lex[ data[p]->l_n ] = p;		
	}

	data.erase( data.end() - 1 );
}

void delete_item_lex( void )
{
	int p = find_lex();

	if ( p == -1 )
	{
		strcpy( result, "NOT FOUND" );
	}
	else
	{
		sprintf( result, "DELETED: %d", data[p]->i_n + 1 );
		real_delete_item(p);
	}
}

void delete_item_inp()
{
	int k = atoi( params ) - 1;

	if ( ( k >= 0 ) && ( k < inp.size() ) )
	{
		sprintf( result, "DELETED: %s", data[ inp[k] ]->desc );
		real_delete_item( inp[k] );
	}
	else
	{
		strcpy( result, "NOT FOUND" );
	}
}

void delete_item_inp_lex()
{
	int k = atoi( params ) - 1;

	if ( ( k >= 0 ) && ( k < lex.size() ) )
	{
		sprintf( result, "DELETED: %s", data[ lex[k] ]->desc );
		real_delete_item( lex[k] );
	}
	else
	{
		strcpy( result, "NOT FOUND" );
	}
}