program catalog;

type
  TCatalogItem = record
    Name: string;
    Index: Integer;
  end;

  TCatalogItemList = array[0..MaxInt div SizeOf(TCatalogItem) - 1] of TCatalogItem;
  PCatalogItemList = ^TCatalogItemList;

  TCommand = (ctUnknown, ctADD, ctFS, ctFN, ctFL, ctDS, ctDN, ctDL);

function CompareStr(const S1, S2: string): Integer;
asm
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,EAX
        MOV     EDI,EDX
        OR      EAX,EAX
        JE      @@1
        MOV     EAX,[EAX-4]
@@1:    OR      EDX,EDX
        JE      @@2
        MOV     EDX,[EDX-4]
@@2:    MOV     ECX,EAX
        CMP     ECX,EDX
        JBE     @@3
        MOV     ECX,EDX
@@3:    CMP     ECX,ECX
        REPE    CMPSB
        JE      @@4
        MOVZX   EAX,BYTE PTR [ESI-1]
        MOVZX   EDX,BYTE PTR [EDI-1]
@@4:    SUB     EAX,EDX
        POP     EDI
        POP     ESI
end;

function ParseLine(const Line: string; var Command: TCommand;
  var IntParam: Integer; var StrParam: string): Boolean;
var
  S: string;
  P, I: Integer;
begin
  Result := False;
  Command := ctUnknown;
  IntParam := 0;
  StrParam := '';
  P := Pos(' ', Line);
  if (P < 3) or (P > 4) then Exit;

  S := Copy(Line, 1, P - 1);

  I := CompareStr(S, 'ADD');
  if I = 0 then
    Command := ctADD
  else
    if I > 0 then
    begin
      I := CompareStr(S, 'DL');
      if I = 0 then
        Command := ctDL
      else
        if I > 0 then
        begin
          I := CompareStr(S, 'DN');
          if I = 0 then
            Command := ctDN
          else
            if I > 0 then
            begin
              I := CompareStr(S, 'DS');
              if I = 0 then
                Command := ctDS
              else
                if I > 0 then
                begin
                  I := CompareStr(S, 'FL');
                  if I = 0 then
                    Command := ctFL
                  else
                    if I > 0 then
                    begin
                      I := CompareStr(S, 'FN');
                      if I = 0 then
                        Command := ctFN
                      else
                        if CompareStr(S, 'FS') = 0 then
                          Command := ctFS;
                    end;
                end;
            end;
        end;
    end;
    
  if Command = ctUnknown then Exit;
  StrParam := Copy(Line, P + 1, MaxInt);
  Result := True;
  if Command in [ctADD, ctFS, ctDS] then Exit;
  Val(StrParam, IntParam, I);
  if I > 0 then Result := False;
end;

var
  CatList: PCatalogItemList;
  Count, Capacity: Integer;

function FindName(const Name: string; var LexIndex: Integer): Boolean;
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    C := CompareStr(CatList^[I].Name, Name);
    if C < 0 then
      L := I + 1
    else begin
      if C = 0 then
      begin
        Result := True;
        L := I;
        Break;
      end;
      H := I - 1;
    end;
  end;
  LexIndex := L;
end;

function FindIndex(const Index: Integer; var LexIndex: Integer): Boolean;
var
  I: Integer;
begin
  Result := False;
  if (Index < 1) or (Index > Count) then Exit;

  for I := 0 to Count - 1 do
    if CatList^[I].Index = Index then
    begin
      LexIndex := I;
      Result := True;
      Break;
    end;
end;

procedure Insert(const LexIndex: Integer; const Name: string; const Index: Integer);
begin
  if Count = Capacity then
  begin
    if Capacity = 0 then
      Capacity := 1000
    else
      Capacity := Capacity + Capacity div 4;
    ReallocMem(CatList, Capacity * SizeOf(TCatalogItem));
  end;

  if LexIndex < Count then
    Move(CatList^[LexIndex], CatList^[LexIndex + 1],
      (Count - LexIndex) * SizeOf(TCatalogItem));

  Pointer(CatList^[LexIndex].Name) := nil;
  CatList^[LexIndex].Index := Index;
  CatList^[LexIndex].Name := Name;
  Inc(Count);
end;

procedure Remove(const LexIndex: Integer);
var
  I, J: Integer;
begin
  I := CatList^[LexIndex].Index;
  Finalize(CatList^[LexIndex]);
  Dec(Count);

  if LexIndex < Count then
    Move(CatList^[LexIndex + 1], CatList^[LexIndex],
      (Count - LexIndex) * SizeOf(TCatalogItem));

  for J := 0 to Count - 1 do
    if CatList^[LexIndex].Index > I then
      Dec(CatList^[LexIndex].Index);
end;


var
  InFile, OutFile: TextFile;
  Command: TCommand;
  StrParam, S: string;
  IntParam, LexIndex: Integer;
begin //WinMain
  AssignFile(InFile, 'catalog.inp');
  Reset(InFile);
  AssignFile(OutFile, 'catalog.out');
  Rewrite(OutFile);

  while not Eof(InFile) do
  begin
    Readln(InFile, S);
    if ParseLine(S, Command, IntParam, StrParam) then
    begin
      case Command of
        ctAdd:
          if not FindName(StrParam, LexIndex) then
          begin
            Insert(LexIndex, StrParam, Count + 1);
            Writeln(OutFile, 'ADDED: ' + StrParam);
          end else
            Writeln(OutFile, 'DUPLICATES NOT ALLOWED');

        ctFS:
          if FindName(StrParam, LexIndex) then
          begin
            Str(CatList^[LexIndex].Index, S);
            Writeln(OutFile, 'FOUND: ' + S);
          end else
            Writeln(OutFile, 'NOT FOUND');

        ctFN:
          if FindIndex(IntParam, LexIndex) then
            Writeln(OutFile, 'FOUND: ' + CatList^[LexIndex].Name)
          else
            Writeln(OutFile, 'NOT FOUND');

        ctFL:
          if (IntParam > 0) and (IntParam <= Count) then
            Writeln(OutFile, 'FOUND: ' + CatList^[IntParam - 1].Name)//LexIndex - zero based
          else
            Writeln(OutFile, 'NOT FOUND');

        ctDS:
          if FindName(StrParam, LexIndex) then
          begin
            Str(CatList^[LexIndex].Index, S);
            Remove(LexIndex);
            Writeln(OutFile, 'DELETED: ' + S);
          end else
            Writeln(OutFile, 'NOT FOUND');

        ctDN:
          if FindIndex(IntParam, LexIndex) then
          begin
            Writeln(OutFile, 'DELETED: ' + CatList^[LexIndex].Name);
            Remove(LexIndex);
          end else
            Writeln(OutFile, 'NOT FOUND');

        ctDL:
          if (IntParam > 0) and (IntParam <= Count) then
          begin
            Writeln(OutFile, 'DELETED: ' + CatList^[IntParam - 1].Name);//LexIndex - zero based
            Remove(IntParam - 1);
          end else
            Writeln(OutFile, 'NOT FOUND');
      end;
    end else
      Writeln(OutFile, 'SYNTAX ERROR');
  end;

  CloseFile(InFile);
  CloseFile(OutFile);

  if Count <> 0 then
    Finalize(CatList^[0], Count * SizeOf(TCatalogItem));

  ReallocMem(CatList, 0);
end.
