/*

VIII PC Magazine Bulgaria programming contest
Problem 1: CATALOG

Author: Peter Ivanov Petrov, 21, Sofia
        software developer / student
        e-mail: piettropro@bigfoot.com


VIII     PC Magazine Bulgaria
 1: 

:   , 21 ., . 
        / 
       e-mail: piettropro@bigfoot.com



  
---------------------

   AVL-,  ,
..        .  
   -      
  ,   -  . 
         O(log N),
 N      .

-    C ,    C++   
- .    .  
  - ,    inline, 
  STL.     STL    
 AVL-,       , 
   .      
,       , 
    (   -  
   STL).

     Microsoft Visual C++ 6.0, Borland
C++ 5.5, MinGW GCC 2.95.2-1  Windows  GCC 2.95.2  Linux. 
EXE-,      Visual C++,    
 C++ ,    ,    
SGI (http://www.stlport.org),    -.

P.S.    ,  .  , 
     .    
  new  delete,   ,  
.       
  ADD, DS, DN  DL,    - 
, -      -
 swapping  .

*/

#include <stdio.h>
#include <stdlib.h>
#include <functional>
#include <memory>
#include <string>
using namespace std;


///////////////////////////////////
// Simple memory allocator class:

template <class T> 
class pp_allocator {
public:
	static T *allocate() { T *p=alloc.allocate(1,hint); hint=p; return p; }
	static void deallocate(T *p) { if (hint==p) hint=NULL; alloc.deallocate(p,1); }
private:
	static std::allocator<T> alloc;
	static T *hint;
};
template <class T> std::allocator<T> pp_allocator<T>::alloc;
template <class T> T *pp_allocator<T>::hint=NULL;


///////////////////////////////////////////////////////////
// Static stack class (faster than the dynamic STL stack)

template <class E,const int MAX_SIZE=128>
class static_stack {
public:
	static_stack() { count=0; }
	bool empty() const { return count==0; }
	int size() const { return count; }
	E& top() { return store[count-1]; }
	const E& top() const { return store[count-1]; }
	void push(const E& x) { store[count++]=x; }
	void pop() { --count; }
private:
	E store[MAX_SIZE];
	int count;
};


///////////////////////////
// Indexed AVL Tree class

template <class E>
class index_tree_node {
public:
	typedef index_tree_node<E> self_type;

	E data;
	self_type *l,*r;
	int cl;
	int bf;

	index_tree_node(const E& e=E()) : data(e) { l=r=NULL; cl=1; bf=0; }

	void *operator new(size_t nBytes) { return pp_allocator<self_type>::allocate(); }
	void operator delete(void *p) { pp_allocator<self_type>::deallocate((self_type*)p); }
};

template <class E,class Pred=std::less<E> >
class index_tree {
public:
	typedef index_tree<E,Pred> self_type;
	typedef index_tree_node<E> node_type;

	index_tree() { root=NULL; count=0; }
	~index_tree() { erase(root); }

	int size() const { return count; }

	bool insert(const E& e) {
		node_type *p=root,*pp=NULL,*a=NULL,*pa;
		while(p) {
			if (p->bf) { a=p; pa=pp; } pp=p;
			if (Pred()(e,p->data)) { ++p->cl; p=p->l; }	else 
				if (Pred()(p->data,e)) p=p->r; else { reset_c(-1,e); return false; }
		}
		node_type *r=new node_type(e);
		if (root) {	if (Pred()(e,pp->data)) pp->l=r; else pp->r=r; } else { root=r; ++count; return true; }
		if (a) {
			if (a->bf==-1)
				if (Pred()(e,a->data)) { a->bf=0; fix_bf(a->l,r,e); } 
				else {
					node_type *b=a->r;
					if (Pred()(b->data,e)) { fix_bf(b->r,r,e); rotate_rr(pa,a,b); } 
					else { fix_bf(b->l,r,e); rotate_rl(pa,a,b);	}
				}
			else 
				if (Pred()(a->data,e)) { a->bf=0; fix_bf(a->r,r,e); }
				else {
					node_type *b=a->l;
					if (Pred()(e,b->data)) { fix_bf(b->l,r,e); rotate_ll(pa,a,b); }
					else { fix_bf(b->r,r,e); rotate_lr(pa,a,b); }
				}
		} else fix_bf(root,r,e);
		++count; return true;
	}

	bool erase(int k,E *e=NULL) {
		if (k<0 || k>=count) return false; ++k;
		static_stack<node_type*,128> st;
		node_type *p=root;
		while(p && p->cl!=k) { st.push(p); if (k<p->cl) { --p->cl; p=p->l; } else { k-=p->cl; p=p->r; } }
		if (!p) { p=root; while(p) if (k<p->cl) { --p->cl; p=p->l; } else { k-=p->cl; p=p->r; } return false; }
		--count; if (e) *e=p->data;
		if (p->l && p->r) {
			node_type *s=p->l;
			st.push(p); while(s->r) { st.push(s); s=s->r; }
			p->data=s->data; --p->cl; p=s;
		}
		node_type *c;
		if (p->l) c=p->l; else c=p->r;
		if (p==root) root=c; else if (p==st.top()->l) st.top()->l=c; else st.top()->r=c;
		E f=p->data; delete p; if (st.empty()) return true;
		node_type *q=st.top(); st.pop();
		while(q) 
			if (!Pred()(q->data,f)) {
				--q->bf; if (q->bf==-1) return true;
				if (q->bf<-1) {
					node_type *b=q->r,*pa;
					if (st.empty()) pa=NULL; else { pa=st.top(); st.pop(); }
					switch(b->bf) {
					case 0: rotate_rr(pa,q,b); b->bf=1; q->bf=-1; return true;
					case 1: rotate_rl(pa,q,b); break;
					default: rotate_rr(pa,q,b);
					}
					q=pa;
				} else { if (st.empty()) return true; q=st.top(); st.pop(); }
			} else {
				++q->bf; if (q->bf==1) return true;
				if (q->bf>1) {
					node_type *b=q->l,*pa;
					if (st.empty()) pa=NULL; else { pa=st.top(); st.pop(); }
					switch(b->bf) {
					case 0: rotate_ll(pa,q,b); b->bf=-1; q->bf=1; return true;
					case 1: rotate_ll(pa,q,b); break;
					default: rotate_lr(pa,q,b);
					}
					q=pa;
				} else { if (st.empty()) return true; q=st.top(); st.pop(); }
			}
		return true;
	}

	int search(const E& e,E *exact=NULL) const { // index, or -1 if not found
		int k=0; node_type *p=root;
		while(p) if (Pred()(e,p->data)) p=p->l; else if (Pred()(p->data,e)) { k+=p->cl; p=p->r;  }
			else { if(exact) *exact=p->data; return k+p->cl-1; }
		return -1;
	}

	bool at(int k,E *e) const {
		if (k<0 || k>=count) return false; ++k;
		node_type *p=root;
		while(p) if (k<p->cl) p=p->l; else if (k>p->cl) { k-=p->cl; p=p->r; } else { if (e) *e=p->data; return true;}
		return false;
	}

protected:
	node_type *root;
	int count;

	void erase(node_type *p) { if (p) { erase(p->l); erase (p->r); delete p; } }

	void fix_bf(node_type *q,node_type *r,const E& e) {
		while(q!=r) if (Pred()(e,q->data)) { q->bf=1; q=q->l; } else { q->bf=-1; q=q->r; }
	}

	void rotate_ll(node_type *pa,node_type *a,node_type *b) {
		a->l=b->r; b->r=a; a->cl-=b->cl;
		if (pa) if (a==pa->l) pa->l=b; else pa->r=b; else root=b;
		a->bf=b->bf=0;
	}

	void rotate_rr(node_type *pa,node_type *a,node_type *b) {
		a->r=b->l; b->l=a; b->cl+=a->cl;
		if (pa) if (a==pa->l) pa->l=b; else pa->r=b; else root=b;
		a->bf=b->bf=0;
	}

	void rotate_lr(node_type *pa,node_type *a,node_type *b) {
		node_type *c=b->r;
		a->l=c->r; b->r=c->l; c->l=b; c->r=a; a->cl-=b->cl+c->cl; c->cl+=b->cl;
		if (pa) if (a==pa->l) pa->l=c; else pa->r=c; else root=c;
		if (c->bf==0) b->bf=a->bf=0; else { if (c->bf==1) { b->bf=0; a->bf=-1; } else { b->bf=1; a->bf=0; }	c->bf=0; }
	}

	void rotate_rl(node_type *pa,node_type *a,node_type *b) {
		node_type *c=b->l;
		a->r=c->l; b->l=c->r; c->l=a; c->r=b; b->cl-=c->cl; c->cl+=a->cl;
		if (pa) if (a==pa->l) pa->l=c; else pa->r=c; else root=c;
		if (c->bf==0) b->bf=a->bf=0; else { if (c->bf==1) { b->bf=-1; a->bf=0; } else { b->bf=0; a->bf=1; }	c->bf=0; }
	}

	void reset_c(int r,const E& e) {
		node_type *p=root;
		while(p) if (Pred()(e,p->data)) { p->cl+=r; p=p->l; } else if (Pred()(p->data,e)) p=p->r; else return;
	}

};


//////////////////////
// Global constants:

const char *IN_FILENAME="CATALOG.INP";
const char *OUT_FILENAME="CATALOG.OUT";


/////////////////////////
// Global declarations:

class Elem {
public:
	string name;
	long id;
	Elem(const string& s,long id) : name(s) { this->id=id; }

	typedef Elem self_type;
	void *operator new(size_t nBytes) { return pp_allocator<self_type>::allocate(); }
	void operator delete(void *p) { pp_allocator<self_type>::deallocate((self_type*)p); }
};

struct lessElemId : public binary_function<Elem*,Elem*,bool> {
	bool operator()(Elem *x,Elem *y) { return x->id<y->id; }
};

struct lessElemLex : public binary_function<Elem*,Elem*,bool> {
	bool operator()(Elem *x,Elem *y) { return x->name<y->name; }
};

typedef index_tree<Elem*,lessElemId> Store;
typedef index_tree<Elem*,lessElemLex> StoreLex;


//////////////////////
// Global variables:

FILE *inF,*outF;
Store store;
StoreLex storeLex;
long storeId;


////////////////////////////
// Miscellanous functions:

void outResult(const string &s) { fputs((s+'\n').c_str(),outF); }
void outError() { outResult("SYNTAX ERROR"); }
bool toInt(string s,int *n) { char c; return sscanf(s.c_str(),"%d%c",n,&c)==1; }
inline string toString(int n) {	char buf[16]; itoa(n,buf,10); return buf; }


/////////////////////////
// Commands processing:

void cmdAdd(string name) {
	Elem *elem=new Elem(name,++storeId);
	if (!storeLex.insert(elem)) { delete elem; outResult("DUPLICATES NOT ALLOWED"); return; }
	store.insert(elem);
	outResult("ADDED: "+name);
}

void cmdFind(string name) {
	Elem elem(name,0),*e;
	int indexLex=storeLex.search(&elem,&e);
	if (indexLex==-1) { outResult("NOT FOUND"); return; }
	outResult("FOUND: "+toString(store.search(e)+1));
}

void cmdDelete(string name) {
	Elem elem(name,0),*e;
	int indexLex=storeLex.search(&elem,&e);
	if (indexLex==-1) { outResult("NOT FOUND"); return; }
	int index=store.search(e);
	outResult("DELETED: "+toString(index+1));
	store.erase(index); storeLex.erase(indexLex); delete e;
}

void cmdFindNum(int n) {
	Elem *e;
	if (!store.at(n-1,&e)) { outResult("NOT FOUND"); return; }
	outResult("FOUND: "+e->name);
}

void cmdDeleteNum(int n) {
	Elem *e;
	if (!store.at(n-1,&e)) { outResult("NOT FOUND"); return; }
	outResult("DELETED: "+e->name);
	store.erase(n-1); storeLex.erase(storeLex.search(e)); delete e;
}

void cmdFindLex(int n) {
	Elem *e;
	if (!storeLex.at(n-1,&e)) { outResult("NOT FOUND"); return; }
	outResult("FOUND: "+e->name);
}

void cmdDeleteLex(int n) {
	Elem *e;
	if (!storeLex.at(n-1,&e)) { outResult("NOT FOUND"); return; }
	outResult("DELETED: "+e->name);
	store.erase(store.search(e)); storeLex.erase(n-1); delete e;
}

void processCmd(string cmd,string param) {
	int iparam;
	if (cmd=="ADD") cmdAdd(param);
	else if (cmd=="FS") cmdFind(param);
	else if (cmd=="FN") { if (!toInt(param,&iparam)) outError(); else cmdFindNum(iparam); }
	else if (cmd=="FL") { if (!toInt(param,&iparam)) outError(); else cmdFindLex(iparam); }
	else if (cmd=="DS") cmdDelete(param);
	else if (cmd=="DN") { if (!toInt(param,&iparam)) outError(); else cmdDeleteNum(iparam); }
	else if (cmd=="DL") { if (!toInt(param,&iparam)) outError(); else cmdDeleteLex(iparam); }
	else outError();
}


///////////////////
// Main function:

void __cdecl main() {
	enum { BUF_SIZE=512 };
	char buf[BUF_SIZE],bufCmd[BUF_SIZE],bufParam[BUF_SIZE];
	int bufPos,lenCmd,lenParam;
	if (!(inF=fopen(IN_FILENAME,"rt"))) abort();
	setvbuf(inF,NULL,_IOFBF,32768);
	if (!(outF=fopen(OUT_FILENAME,"wt"))) abort();
	setvbuf(outF,NULL,_IOFBF,32768);
	// Main loop:
	storeId=0;
	while (!feof(inF)) {
		char c;
		if (!fgets(buf,BUF_SIZE,inF)) break;
		if (buf[0]==0 || buf[0]=='\n') continue;
		bufPos=lenCmd=0;
		while ((c=buf[bufPos++])!=' ' && c!=0) bufCmd[lenCmd++]=c;
		bufCmd[lenCmd]=lenParam=0;
		if (c!=' ') { outError(); continue; }
		while ((c=buf[bufPos++]) && (c!='\n')) bufParam[lenParam++]=c;
		if (lenParam<1 || bufParam[lenParam-1]==' ' || bufParam[0]==' ') { outError(); continue; }
		bufParam[lenParam]=0;
		processCmd(bufCmd,bufParam);
	}	
	// Final stuff:
	fclose(inF);
	fclose(outF);
}
