program catalog;

uses
  SySUtils;

{$APPTYPE CONSOLE}

const
  inpf = 'catalog.inp';
  outf  = 'catalog.out';
  maxcnt = 300000;

type
  database = record
               name : string;
               inpn : integer;
             end;

var
  db             : array[1..maxcnt] of database;
  input,output   : text;
  ccom,cpar      : string;
  dbcnt          : integer;
  addl,nums,lexs : boolean;

procedure init;
begin
  assign(input,inpf);
  assign(output,outf);
  reset(input);
  rewrite(output);
end;

procedure swap(var El1,El2 : database);
var
  Tmp : database;
begin
  Tmp := El1;
  El1 := El2;
  El2 := Tmp;
end;

procedure intsort(L,R : integer);
var
  I,J,X : integer;
begin
  I := L; J := R; X := db[(I+J) div 2].inpn;
  repeat
    while X > db[I].inpn do Inc(I);
    while X < db[J].inpn do Dec(J);
    if I <= J then begin
      swap(db[I],db[J]);
      Inc(I); Dec(J);
    end;
  until I > J;
  if J > L then intsort(L,J);
  if I < R then intsort(I,R);
  nums := true;
  lexs := false;
  addl := false;
end;

procedure lexsort(L,R : integer);
var
  I,J : integer;
  X : string;
begin
  I := L; J := R; X := db[(I+J) div 2].name;
  repeat
    while X > db[I].name do Inc(I);
    while X < db[J].name do Dec(J);
    if I <= J then begin
      swap(db[I],db[J]);
      Inc(I); Dec(J);
    end;
  until I > J;
  if J > L then lexsort(L,J);
  if I < R then lexsort(I,R);
  lexs := true;
  nums := false;
  addl := false;
end;

procedure add;
var
  i : integer;
begin
  for i := 1 to dbcnt do begin
    if db[i].name = cpar then begin
      writeln(output,'DUPLICATES NOT ALLOWED');
      exit;
    end;
  end;
  dbcnt := dbcnt + 1;
  db[dbcnt].name := cpar;
  db[dbcnt].inpn := dbcnt;
  addl := true;
  writeln(output,'ADDED: '+cpar);
end;

procedure fs;
var
  i : integer;
begin
  for i := 1 to dbcnt do begin
    if db[i].name = cpar then begin
      writeln(output,'FOUND: '+inttostr(db[i].inpn));
      exit;
    end;
  end;
  writeln(output,'NOT FOUND');
end;

procedure fn;
var
  i : integer;
begin
  i := strtoint(cpar);
  if (i < 0) or (i > dbcnt) then begin
    writeln(output,'NOT FOUND');
    exit;
  end;
  if addl or lexs then intsort(1,dbcnt);
  writeln(output,'FOUND: '+db[i].name);
end;

procedure fl;
var
  i : integer;
begin
  i := strtoint(cpar);
  if (i < 0) or (i > dbcnt) then begin
    writeln(output,'NOT FOUND');
    exit;
  end;
  if addl or nums then lexsort(1,dbcnt);
  writeln(output,'FOUND: '+db[i].name);
end;

procedure ds;
var
  i : integer;
  fnd : boolean;
begin
  fnd := false;
  if addl or lexs then intsort(1,dbcnt);
  for i := 1 to dbcnt do begin
    if db[i].name = cpar then begin
      fnd := true;
      writeln(output,'DELETED: '+inttostr(db[i].inpn));
    end;
    if fnd then begin
      db[i] := db[i+1];
      dec(db[i].inpn);
    end;
  end;
  if fnd then dbcnt := dbcnt - 1
  else writeln(output,'NOT FOUND');
end;

procedure dn;
var
  i,j : integer;
begin
  i := strtoint(cpar);
  if (i < 1) or (i > dbcnt) then begin
    writeln(output,'NOT FOUND');
    exit;
  end;
  if addl or lexs then intsort(1,dbcnt);
  writeln(output,'DELETED: '+db[i].name);
  dbcnt := dbcnt - 1;
  for j := i to dbcnt do begin
    db[j] := db[j+1];
    dec(db[j].inpn);
  end;
end;

procedure dl;
var
  i,j,k : integer;
begin
  i := strtoint(cpar);
  if (i < 1) or (i > dbcnt) then begin
    writeln(output,'NOT FOUND');
    exit;
  end;
  if addl or nums then lexsort(1,dbcnt);
  writeln(output,'DELETED: '+db[i].name);
  k := db[i].inpn;
  dbcnt := dbcnt - 1;
  for j := 1 to dbcnt do begin
    if j >= i then db[j] := db[j+1];
    if k < db[j].inpn then dec(db[j].inpn);
  end;
end;

procedure pars_input_info;
var
  csep : integer;
  com,vpar : string;
begin
  readln(input,com);
  csep := pos(' ',com);
  ccom := copy(com,1,csep-1);
  cpar := copy(com,csep+1,length(com)-csep);
  vpar := copy(cpar,1,1);
  if (ccom <> 'ADD') and (ccom <> 'FS') and (ccom <> 'FN')
  and (ccom <> 'FL') and (ccom <> 'DS') and (ccom <> 'DN')
  and (ccom <> 'DL') then begin
    writeln(output,'SYNTAX ERROR');
    exit;
  end;
  if (vpar = ' ') or (cpar = '') then begin
    writeln(output,'SYNTAX ERROR');
    exit;
  end;
  if ccom = 'ADD' then add;
  if ccom = 'FS' then fs;
  if ccom = 'FN' then fn;
  if ccom = 'FL' then fl;
  if ccom = 'DS' then ds;
  if ccom = 'DN' then dn;
  if ccom = 'DL' then dl;
end;

procedure finish;
begin
  close(input);
  close(output);
end;

begin
  init;
  repeat
    pars_input_info;
  until eof(input);
  finish;
end.
