// Catalog.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <process.h>

#define MAX_ARRSIZE 1000

char buf[256]; // Buffer for the line read from the input file

FILE *OutFile; // The output file - CATALOG.OUT

struct Node;

struct ArrayList {
	unsigned long NumberPreceding; // How many elements do we have before this array

	unsigned long NumberOfElements; // The number of elements in the array
	Node **Element; // The array of elements

	ArrayList *next;
};

struct NodeList {
	Node *p;
	NodeList *next;
	char Symbol;
};

struct Node {
	Node *parent; // We have the parent so that we can easily move up the tree
	char Symbol;  // We add Symbol so that we know which path brought us to this node

	bool IsElement; // Whether our Node is the last symbol of an element
	ArrayList *ArrLst;			 // Pointer to the array our element is in
	unsigned long NumberInArray; // Number of the element in this array

	unsigned long HowManyUnder; // How many elements are under this node in the tree

	NodeList *FirstChildNode; // List of child nodes
};

ArrayList *first,*last;
Node *root;
unsigned long TotalNumber; // Total number of elements

/* Performs the ADD command */
void Add(char *p)
{
	char *pp=p,c;
	Node *np=root;
	NodeList **NList,*TempList;

	/* We walk down the tree, creating nodes as needed */
	while(c=*pp) {
		np->HowManyUnder++; // We have a new element under the current

		NList=&(np->FirstChildNode);

		/* We browse the sorted list of child nodes, searching for the appropriate one */
		while((*NList)&&((*NList)->Symbol<c)) NList=&((*NList)->next);

		/* If there is no appropriate node for the current symbol - we create it */
		if((!(*NList))||(((*NList)->Symbol)>c)) {
			TempList=new NodeList;

			if(!TempList) {						 // Not enough memory
				fputs("NOT ENOUGH MEMORY\n",OutFile);
				exit(1);
			}

			TempList->next=(*NList);
			TempList->Symbol=c;
			TempList->p=0;

			(*NList)=TempList;
		}

		/* If we have just created a list element, we create a new node and attach it */
		if(!((*NList)->p)) {
			(*NList)->p=new Node;

			if(!((*NList)->p)) {				 // Not enough memory
				fputs("NOT ENOUGH MEMORY\n",OutFile);
				exit(1);
			}

			(*NList)->p->parent=np;				 // Set parent

			np=(*NList)->p;

			np->Symbol=c;						 // Set Symbol

			np->FirstChildNode=0;				 // Set FirstChildNode
			np->HowManyUnder=0;					 // Set HowManyUnder

			np->IsElement=false;				 // Set IsElement
			np->ArrLst=0;						 // Set ArrLst
			np->NumberInArray=0;				 // Set NumberInArray
		}
		else np=(*NList)->p;

		pp++;
	}

	/* If the node corresponding is already marked as element, we have a duplicate */
	if(np->IsElement) {
		/* So we print the error message */
		fputs("DUPLICATES NOT ALLOWED\n",OutFile);

		/* And then decrease every HowManyUnder property which we have increased */
		pp=p;
		np=root;

		while(c=*pp) {
			np->HowManyUnder--; // Decrease

			NList=&(np->FirstChildNode);

			while((*NList)&&((*NList)->Symbol<c)) NList=&((*NList)->next);

			np=(*NList)->p;

			pp++;
		}
		
		return;
	}

	/* Now we have to add our new element to the list of elements */

	/* If the list is empty, we create it */
	if(!last) {
		first=last=new ArrayList;

		if(!last) {			// Not enough memory
			fputs("NOT ENOUGH MEMORY\n",OutFile);
			exit(1);
		}

		last->NumberPreceding=0;
		last->next=0;
		last->NumberOfElements=0;

		last->Element=new Node *[MAX_ARRSIZE];

		if(!(last->Element)) { // Not enough memory
			fputs("NOT ENOUGH MEMORY\n",OutFile);
			exit(1);
		}
	}

	/* If the current list node is full, we create a new one and attach it to the list */
	if((last->NumberOfElements)==MAX_ARRSIZE) {
		last->next=new ArrayList;

		if(!(last->next)) {			// Not enough memory
			fputs("NOT ENOUGH MEMORY\n",OutFile);
			exit(1);
		}

		last->next->NumberPreceding=last->NumberPreceding+last->NumberOfElements;
		last->next->next=0;
		last->next->NumberOfElements=0;

		last->next->Element=new Node *[MAX_ARRSIZE];

		if(!(last->next->Element)) { // Not enough memory
			fputs("NOT ENOUGH MEMORY\n",OutFile);
			exit(1);
		}

		last=last->next;
	}

	/* Now we mark our new element as an element and set its properties */
	np->IsElement=true;
	np->ArrLst=last;
	np->NumberInArray=last->NumberOfElements;
	np->HowManyUnder++;

	/* Then we attach it to the list of elements */
	last->Element[last->NumberOfElements++]=np;

	/* And print the results in the output file */
	fputs("ADDED: ",OutFile);
	fputs(p,OutFile);
	fputs("\n",OutFile);

	/* Finally, we increase the total number of elements by 1 */
	TotalNumber++;
}

char PNO_Temp[250]; // PrintNameOf Temp string

/* Prints the string, represented by a given element's node */
void PrintNameOf(Node *np)
{
	unsigned long i=247;

	/* We walk back through the tree and restore the name of the element */
	while(np->parent) {
		PNO_Temp[i--]=np->Symbol;
		np=np->parent;
	}

	/* Then we print it to the output file */
	fputs(PNO_Temp+i+1,OutFile);
}

/* Performs the FS command */
void FindByName(char *p)
{
	Node *np=root;
	NodeList *nlp;
	char c,num[10];

	/* We walk down the tree towards the element we're searching for */
	while(c=*p) {
		nlp=np->FirstChildNode;

		while(nlp&&(nlp->Symbol!=c)) nlp=nlp->next;

		if(!nlp) { // Not Found
			fputs("NOT FOUND\n",OutFile);
			return;
		}

		np=nlp->p;

		p++;
	}

	/* If the node found is not an element */
	if(!(np->IsElement)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* And, if we found an element, print its number */
	fputs("FOUND: ",OutFile);
	fputs(ultoa(np->ArrLst->NumberPreceding+np->NumberInArray+1,num,10),OutFile);
	fputs("\n",OutFile);
}

/* Performs the FN command */
void FindByNumber(char *p)
{
	ArrayList *alp=first;
	Node *np;
	unsigned long num;
	char *endptr;
	
	/* We extract the number from the parameter */
	num=strtoul(p,&endptr,10);

	/* If the parameter is not a valid number, print SYNTAX ERROR */
	if(*endptr) {
		fputs("SYNTAX ERROR\n",OutFile);
		return;
	}

	/* If there is no such number, print error */
	if((!num)||(num-->TotalNumber)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* If there is such a number, we walk through the list of elements */
	while(alp&&(alp->next)&&(alp->next->NumberPreceding<=num)) alp=alp->next;

	/* And get the node, representing our element */
	np=alp->Element[num-alp->NumberPreceding];

	/* Finally, we print the results to the output file */
	fputs("FOUND: ",OutFile);
	PrintNameOf(np);
}

/* Performs the FL command */
void FindByLexicographicalNumber(char *p)
{
	Node *np=root;
	NodeList *nlp;
	unsigned long num;
	char *endptr;
	
	/* We extract the number from the parameter */
	num=strtoul(p,&endptr,10);

	/* If the parameter is not a valid number, print SYNTAX ERROR */
	if(*endptr) {
		fputs("SYNTAX ERROR\n",OutFile);
		return;
	}

	/* If there is no such number, print error */
	if((!num)||(num>TotalNumber)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* We walk down the tree towards the element we're searching for */
	while(num) {
		nlp=np->FirstChildNode;

		while(nlp&&((nlp->p->HowManyUnder)<num)) {
			num-=nlp->p->HowManyUnder;
			nlp=nlp->next;
		}

		np=nlp->p;

		if(np->IsElement) num--;
	}

	/* And, if found, print its name */
	fputs("FOUND: ",OutFile);
	PrintNameOf(np);
}

/* Deletes an element given its representing node */
void DeleteElement(Node *np)
{
	ArrayList *alp,*alp2;
	Node **Element;
	NodeList **nlp,*TempList;
	unsigned long i,ne;
	char c;

	alp=np->ArrLst;
	
	/* First we fix the numbers in the list of elements */
	alp->NumberOfElements--;
	while(alp=alp->next) alp->NumberPreceding--;

	alp=np->ArrLst;

	/* Then we remove the element from the list and fix the indexes of the elements after */
	for(i=np->NumberInArray,ne=alp->NumberOfElements,Element=alp->Element;i<ne;i++) {
		Element[i]=Element[i+1];
		Element[i]->NumberInArray--;
	}

	/* If we don't need this List node any more, we delete it */
	if(!(alp->NumberOfElements)) {
		/* If we delete the first node */
		if(alp==first) {
			first=alp->next;

			delete alp->Element;
			delete alp;
		}
		else {
			/* We do not delete the first -> we have to find it */
			alp2=first;

			while((alp2->next)!=alp) alp2=alp2->next;

			alp2->next=alp->next;

			delete alp->Element;
			delete alp;
		}
	}

	/* And actually mark the element as not being an element */
	np->IsElement=false;

	/* Then, if necessary, we remove not-needed nodes */
	while((!(np->FirstChildNode))&&(!(np->IsElement))&&(np->parent)) {
		/* Doesn't have children, is not an element and is not the root, so it's not-needed */
		c=np->Symbol;

		np=np->parent;

		nlp=&(np->FirstChildNode);

		/* We search for the link to the not-needed node */
		while(((*nlp)->Symbol)<c) nlp=&((*nlp)->next);

		/* We detach the link from the list of links */
		TempList=(*nlp);
		*nlp=(*nlp)->next;

		/* Then we delete the non-needed node */
		delete TempList->p;

		/* And the link itself */
		delete TempList;
	}

	/* And finally, we go up the tree and fix the HowManyUnder properties */
	do {
		np->HowManyUnder--;
		np=np->parent;
	} while(np);

	/* And do not forget to decrease the total number of elements */
	TotalNumber--;
}

/* Performs the DS command */
void DeleteByName(char *p)
{
	Node *np=root;
	NodeList *nlp;
	char c,num[10];

	/* We walk down the tree towards the element we're searching for */
	while(c=*p) {
		nlp=np->FirstChildNode;

		while(nlp&&(nlp->Symbol!=c)) nlp=nlp->next;

		if(!nlp) { // Not Found
			fputs("NOT FOUND\n",OutFile);
			return;
		}

		np=nlp->p;

		p++;
	}

	/* If the node found is not an element */
	if(!(np->IsElement)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* If we found the element print its number */
	fputs("DELETED: ",OutFile);
	fputs(ultoa(np->ArrLst->NumberPreceding+np->NumberInArray+1,num,10),OutFile);
	fputs("\n",OutFile);

	/* Then we actually delete the element */
	DeleteElement(np);
}

/* Performs the DN command */
void DeleteByNumber(char *p)
{
	ArrayList *alp=first;
	Node *np;
	unsigned long num;
	char *endptr;
	
	/* We extract the number from the parameter */
	num=strtoul(p,&endptr,10);

	/* If the parameter is not a valid number, print SYNTAX ERROR */
	if(*endptr) {
		fputs("SYNTAX ERROR\n",OutFile);
		return;
	}

	/* If there is no such number, print error */
	if((!num)||(num-->TotalNumber)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* If there is such a number, we walk through the list of elements */
	while(alp&&(alp->next)&&(alp->next->NumberPreceding<=num)) alp=alp->next;

	/* And get the node, representing our element */
	np=alp->Element[num-alp->NumberPreceding];

	/* We print the results to the output file */
	fputs("DELETED: ",OutFile);
	PrintNameOf(np);

	/* And finally delete the element */
	DeleteElement(np);
}

/* Performs the DL command */
void DeleteByLexicographicalNumber(char *p)
{
	Node *np=root;
	NodeList *nlp;
	unsigned long num;
	char *endptr;
	
	/* We extract the number from the parameter */
	num=strtoul(p,&endptr,10);

	/* If the parameter is not a valid number, print SYNTAX ERROR */
	if(*endptr) {
		fputs("SYNTAX ERROR\n",OutFile);
		return;
	}

	/* If there is no such number, print error */
	if((!num)||(num>TotalNumber)) {
		fputs("NOT FOUND\n",OutFile);
		return;
	}

	/* We walk down the tree towards the element we're searching for */
	while(num) {
		nlp=np->FirstChildNode;

		while(nlp&&((nlp->p->HowManyUnder)<num)) {
			num-=nlp->p->HowManyUnder;
			nlp=nlp->next;
		}

		np=nlp->p;

		if(np->IsElement) num--;
	}

	/* Print the results */
	fputs("DELETED: ",OutFile);
	PrintNameOf(np);

	/* And actually delete the element */
	DeleteElement(np);
}

/* Reads the input file command by command and executes the commands */
int ReadData()
{
	FILE *f=fopen("CATALOG.INP","rt");
	char *p,*p2;

	if(!f) return 1; // We couldn't open the input file

	OutFile=fopen("CATALOG.OUT","wt");

	if(!OutFile) return 1; // We couldn't open the output file

	/* Init */
	root=new Node;
	if(!root) return 1;
	memset(root,0,sizeof(Node));
	PNO_Temp[249]=0;
	PNO_Temp[248]='\n';

	while(fgets(buf,255,f)) { // For every read line
		p2=strchr(buf,'\n');   // We remove the newline character
		if(p2) *p2=0;
		else p2=strchr(buf,0);

		p=strchr(buf,' ');    // And search for the space between the command and the parameter
		if(!p) {
			fputs("SYNTAX ERROR\n",OutFile); // If there is no space - Syntax Error
			continue;
		}

		*(p++)=0; // We separate the command from the parameter

		if((!(*p))||((*p)==' ')||((*(p2-1))==' ')) {
			fputs("SYNTAX ERROR\n",OutFile);
			continue;
		}

		if((buf[0]=='A')&&(!buf[3])) {
			if((buf[1]!='D')||(buf[2]!='D')) fputs("SYNTAX ERROR\n",OutFile);
			else Add(p);
		}
		else if((buf[0]=='F')&&(!buf[2])) {
			if(buf[1]=='S') FindByName(p);
			else if(buf[1]=='N') FindByNumber(p);
			else if(buf[1]=='L') FindByLexicographicalNumber(p);
			else fputs("SYNTAX ERROR\n",OutFile);
		}
		else if((buf[0]=='D')&&(!buf[2])) {
			if(buf[1]=='S') DeleteByName(p);
			else if(buf[1]=='N') DeleteByNumber(p);
			else if(buf[1]=='L') DeleteByLexicographicalNumber(p);
			else fputs("SYNTAX ERROR\n",OutFile);
		}
		else fputs("SYNTAX ERROR\n",OutFile);
	}

	fclose(OutFile);
	fclose(f);

	return 0;
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	return ReadData();
}



