#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>

struct ListItem {
  char* name;
  long lexNum;
  long posNum;
};

class Catalogue {
  private :
    ListItem** elements;
    ListItem** sorted;
    int size;
    int last;
    int findSuccess;
    int findPos;
    char temp[255] ;
    void resize();
    void remove(int, int);
  public :
    Catalogue();
    char* add(char*);
    char* remove(char*);
    char* removePositional(int position);
    char* removeLexical(int position);
    char* find(char* name);
    char* findPositional(int position);
    char* findLexical(int position);
};

  char* Catalogue::add(char* name){
    find(name);
    if (findSuccess) {
      delete[] name;
      return "DUPLICATES NOT ALLOWED";
    }
    if(size == (last + 1)){
      resize();
    }
    last++;
    ListItem* newElement = new ListItem;
    newElement -> name = name;
    newElement -> posNum = last;
    newElement -> lexNum = findPos;
    elements[last] = newElement;
    for(int i = last; i>findPos; i--){
      sorted[i]=sorted[i-1];
      sorted[i]->lexNum = i;
    }
    sorted[findPos] = newElement;
    strcpy(temp,"ADDED: ");
    strcat(temp,name);
    return temp;
  }

  char* Catalogue::remove(char* name){
    find(name);
    if(!findSuccess){
      return "NOT FOUND";
    }
    ListItem* element = sorted[findPos];
    remove(element->lexNum,element->posNum);
    strcpy(temp,"DELETED: ");
    char tv[15];
    ltoa(element->posNum+1,tv,10);
    strcat(temp,tv);
    delete[] name;
    delete element;
    return temp;
  }

  char* Catalogue::removePositional(int position){
    if((position<0)||(position>last)){
      return "NOT FOUND";
    }
    ListItem* element = elements[findPos];
    remove(element->lexNum,element->posNum);
    strcpy(temp,"DELETED: ");
    strcat(temp,element->name);
    delete[] element->name;
    delete element;
    return temp;
  }

  char* Catalogue::removeLexical(int position){
    if((position<0)||(position>last)){
      return "NOT FOUND";
    }
    ListItem* element = sorted[position];
    remove(element->lexNum,element->posNum);
    strcpy(temp,"DELETED: ");
    strcat(temp,element->name);
    delete[] element->name;
    delete element;
    return temp;
  }

  void Catalogue::remove(int sortPos, int intPos){
    int i;
    for(i=sortPos;i<last;i++){
      sorted[i]=sorted[i+1];
      sorted[i]->lexNum = i;
    }
    for(i = intPos; i<last; i++){
      elements[i] = elements[i+1];
      elements[i]->posNum = i;
    }
    last--;
  }

  char* Catalogue::findPositional(int position){
    if((position<0)||(position>last)){
      return "NOT FOUND";
    }
    ListItem* element = elements[position];
    strcpy(temp,"FOUND: ");
    strcat(temp,element->name);
    return temp;
  }

  char* Catalogue::findLexical(int position){
    if((position<0)||(position>last)){
      return "NOT FOUND";
    }
    ListItem* element = sorted[position];
    strcpy(temp,"FOUND: ");
    strcat(temp,element->name);
    return temp;
  }

  char* Catalogue::find(char* name){
    int i1 = 0;
    int i2 = last;
    int cmp;
    findPos = 0;
    while(i1<i2){
      cmp = strcmp(sorted[(i1+i2)/2]->name,name);
      if(cmp>0){
	findPos = (i1+i2)/2;
	i2 = (i1+i2)/2 - 1;
      } else if(cmp<0){
	findPos = (i1+i2)/2+1;
	i1 = (i1+i2)/2 + 1;
      } else {
	strcpy(temp,"FOUND: ");
	char tmpstr[15];
	itoa(sorted[(i1+i2)/2]->posNum+1,tmpstr,10);
	strcat(temp,tmpstr);
	findSuccess = 1;
	findPos = (i1+i2)/2;
	return temp;
      }
    }
    findPos = i1;
    cmp=strcmp(sorted[i1]->name,name);
    if(cmp==0){
	strcpy(temp,"FOUND: ");
	char tmpstr[15];
	itoa(sorted[i1]->posNum+1,tmpstr,10);
	strcat(temp,tmpstr);
	findSuccess = 1;
	findPos = (i1+i2)/2;
	return temp;
    } else if(cmp<0){
      findPos=i1+1;
    }
    findSuccess = 0;
    return "NOT FOUND";
  }

  void Catalogue::resize(){
    int i;
    ListItem** tmp = elements;
    elements = new ListItem*[last+100];
    for(i = 0;i<=last;i++){
      elements[i] = tmp[i];
    }
    delete[] tmp;
    tmp = sorted;
    sorted = new ListItem*[last+100];
    for(i=0;i<=last;i++){
      sorted[i] = tmp[i];
    }
    size = last+100;
  }

  Catalogue::Catalogue(){
    elements = new ListItem*[100];
    sorted = new ListItem*[100];
    size = 100;
    last = -1;
  }

  void main(){
    char temp[255];
    char* temp2;

    FILE* inp = fopen("catalog.inp","rt");
    FILE* out = fopen("catalog.out","wt");
    Catalogue cat;
    int has = 0;
    int len;
    while(fgets(temp,255,inp)){
      if(has){
	fprintf(out,"\n");
      }
      has = 1;
      len = strlen(temp);
      if(temp[len-1]=='\n'){
	len--;
	temp[len]='\0';
      }
      if(!strncmp(temp,"ADD ",4)){
	temp2 = new char[len-3];
	strcpy(temp2,temp+4);
	fprintf(out,"%s",cat.add(temp2));
      } else if(!strncmp(temp,"FS ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.find(temp2));
      } else if(!strncmp(temp,"FN ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.findPositional(atol(temp2)-1));
      } else if(!strncmp(temp,"FL ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.findLexical(atol(temp2)-1));
      } else if(!strncmp(temp,"DS ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.remove(temp2));
      } else if(!strncmp(temp,"DN ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.removePositional(atol(temp2)-1));
      } else if(!strncmp(temp,"DL ",3)){
	temp2 = new char[len-2];
	strcpy(temp2,temp+3);
	fprintf(out,"%s",cat.removeLexical(atol(temp2)-1));
      } else {
	fprintf(out,"SYNTAX ERROR");
      }
    }
    fclose(inp);
    fclose(out);
  }
