#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <fstream.h>
#include <io.h>
#include <fcntl.h>

FILE *fin;
FILE *fout;

char *ADDED = "ADDED: ";
int ADDED_LEN = 7;

char *FOUND = "FOUND: ";
int FOUND_LEN = 7;

char *DELETED = "DELETED: ";
int DELETED_LEN = 9;

char *NOT_FOUND = "NOT FOUND\n";
char *SYNTAX_ERROR = "SYNTAX ERROR\n";
char *DUPLICATES = "DUPLICATES NOT ALLOWED\n";

struct node {
  char *string;
  node *left;
  node *right;
  int lcount;
  int rcount;
  int prior;
  int number;
};

typedef node *pnode;

char found, cmpStrings, *lastString, nocompare=0;
int stockCount, counter, lastNumber, lastIndex;
char stringBuffer[301];
pnode tree, tree2;

inline pnode makeNode(char *s, pnode left, pnode right, int lcount, int rcount, int number) {
  pnode temp = new node;
  temp->string = s;
  temp->left = left;
  temp->right = right;
  temp->prior = rand();
  temp->lcount = lcount;
  temp->rcount = rcount;
  temp->number = number;
  return temp;
}

inline pnode rotateLeftChild(pnode k2) {
  pnode k1 = k2->left;
  k2->left = k1->right;
  k1->right = k2;

  k2->lcount = k1->rcount;
  k1->rcount += k2->rcount + 1;

  return k1;
}

inline pnode rotateRightChild(pnode k1 ) {
  pnode k2 = k1->right;
  k1->right = k2->left;
  k2->left = k1;

  k1->rcount = k2->lcount;
  k2->lcount += k1->lcount + 1;

  return k2;
}

inline int compareNodes(pnode t1, char *s, int l) {
  if (nocompare) return -1;
  return cmpStrings ? strcmp(t1->string, s) :
		 t1->number > l ? 1 : t1->number == l ? 0 : -1;
}


void insertString(pnode &tree, char *s, int count) {
  if (tree == NULL) {
	tree = makeNode(s, NULL, NULL, 0, 0, count);
  } else {
	int compare = compareNodes(tree, s, count);

	if (compare < 0) {
	  insertString(tree->left, s, count);
	  if (!found) tree->lcount++;
	  if (tree->left->prior < tree->prior) {
		tree = rotateLeftChild(tree);
	  }
	} else
	if (compare > 0) {
	  insertString(tree->right, s, count);
	  if (!found) tree->rcount++;
	  if (tree->right->prior < tree->prior) {
		tree = rotateRightChild(tree);
	  }
	} else {
	  found = 1;
	}
  }
}

void deleteRoot(pnode &tree) {
  if (tree->left == NULL) {
	pnode temp = tree->right;
	delete tree;
	tree = temp;
	return;
  }

  if (tree->right == NULL) {
	pnode temp = tree->left;
	delete tree;
	tree = temp;
	return;
  }

  if (tree->left->prior < tree->right->prior) {
	tree = rotateLeftChild(tree);
	deleteRoot(tree->right);
  } else {
	tree = rotateRightChild(tree);
	deleteRoot(tree->left);
  }
}


pnode deleteString(pnode tree, char *s, int l) {
  if (tree == NULL) return NULL;

  int compare = compareNodes(tree, s, l);

  if (compare < 0) {
	lastIndex += tree->rcount + 1;
	tree->left = deleteString(tree->left, s, l);
	if (found) (tree->lcount)--;
  } else
  if (compare > 0) {
	tree->right = deleteString(tree->right, s, l);
	if (found) (tree->rcount)--;
  } else {
	found = 1;
	lastNumber = tree->number;
	lastString = tree->string;
	lastIndex += tree->rcount + 1;

	deleteRoot(tree);

  }

  return tree;
}

pnode deleteStringByIndex(pnode tree, int number, int index) {
  if (tree == NULL) return NULL;

  if ((index + tree->rcount + 1 > number)) {
	tree->right = deleteStringByIndex(tree->right, number, index);
	(tree->rcount)--;
  } else
  if ((index + tree->rcount + 1 < number)) {
	index += tree->rcount + 1;
	tree->left = deleteStringByIndex(tree->left, number, index);
	(tree->lcount)--;
  } else {
	found = 1;
	lastNumber = tree->number;
	lastString = tree->string;
	lastIndex += tree->rcount + 1;

	deleteRoot(tree);
  }

  return tree;
}



pnode findString(pnode tree, char *s, int l) {
  int compare;

  while (tree != NULL) {
	compare = compareNodes(tree, s, l);

	if (compare < 0) {
	  tree = tree->left;
	} else
	if (compare > 0) {
	  tree = tree->right;
	} else {
	  return tree;
	}
  }

  return NULL;
}


int findIndexByString(pnode tree, char *s, int l) {
  int compare, index=0;

  while (tree != NULL) {
	compare = compareNodes(tree, s, l);

	if (compare < 0) {
	  index += tree->rcount + 1;
	  tree = tree->left;
	} else
	if (compare > 0) {
	  tree = tree->right;
	} else {
	  return index + tree->rcount + 1;
	}
  }

  return -1;
}


pnode findStringByIndex(pnode tree, int number) {
  int index=0;

  while (tree != NULL) {
	if ((index + tree->rcount + 1 > number)) {
	  tree = tree->right;
	} else
	if ((index + tree->rcount + 1 < number)) {
	  index += tree->rcount + 1;
	  tree = tree->left;
	} else {
	  return tree;
	}
  }

  return NULL;
}



void printtree(pnode t, int level) {
  if (t != NULL) {
	for (int i=0; i<level; i++) printf(" ");
	printf("name: %s prior %d l=%d r=%d\n", t->string, t->prior, t->lcount, t->rcount);
	printtree(t->left, level + 5);
	printtree(t->right, level + 5);
  }
}

void printinfix(pnode t) {
  if (t!=NULL) {
	printinfix(t->left);
	printf("%s ", t->string);
	printinfix(t->right);
  }
}


void addCommand(char *buffer) {
  found = 0;
  cmpStrings = 1;

  char *s = strdup(buffer);

  insertString(tree, s, counter);

  if (found) {
	  fputs(DUPLICATES, fout);
  } else {
	  stockCount++;
	  cmpStrings = 0;

	  /**/nocompare=1;
	  insertString(tree2, s, counter++);
	  /**/nocompare=0;

	  strncpy(stringBuffer, ADDED, ADDED_LEN);
	  strcpy(stringBuffer + ADDED_LEN, buffer);
	  fputs(stringBuffer, fout);
  }
}

void fsCommand(char *buffer) {
  cmpStrings = 1;

  pnode tmp = findString(tree, buffer, 0);

  if (tmp == NULL) {
	fputs(NOT_FOUND, fout);
  } else {
	cmpStrings = 0;

	strncpy(stringBuffer, FOUND, FOUND_LEN);
	sprintf(stringBuffer + FOUND_LEN, "%d\n", findIndexByString(tree2, NULL, tmp->number));
	fputs(stringBuffer, fout);
  }
}

void fnlCommand(char *buffer, pnode tree) {
  pnode tmp = NULL;
  int index = atoi(buffer);

  if (index <= stockCount && index > 0) {
	tmp = findStringByIndex(tree, index);
  }

  if (tmp == NULL) {
	fputs(NOT_FOUND, fout);
  } else {
	strncpy(stringBuffer, FOUND, FOUND_LEN);
	strcpy(stringBuffer + FOUND_LEN, tmp->string);
	fputs(stringBuffer, fout);
  }
}

void dsCommand(char *buffer) {
  found = 0;
  cmpStrings=1;

  lastNumber=0;
  tree = deleteString(tree, buffer, 0);

  if (found) {
	stockCount--;
	cmpStrings=0;
	lastIndex=0;
	tree2 = deleteString(tree2, buffer, lastNumber);
	strncpy(stringBuffer, DELETED, DELETED_LEN);
	sprintf(stringBuffer + DELETED_LEN, "%d\n", lastIndex);
	fputs(stringBuffer, fout);
	free(lastString);
  } else {
	fputs(NOT_FOUND, fout);
  }

}

void dnlCommand(char *buffer, char compare) {
  found=0;
  int index = atoi(buffer);

  cmpStrings = compare;

  if (index <= stockCount && index > 0) {
	lastNumber=0;

	if (cmpStrings) {
	  tree = deleteStringByIndex(tree, index, 0);
	  cmpStrings=0;
	} else {
	  tree2 = deleteStringByIndex(tree2, index, 0);
	  cmpStrings=1;
	}
  }

  if (found) {
	stockCount--;
	strncpy(stringBuffer, DELETED, DELETED_LEN);
	strcpy(stringBuffer + DELETED_LEN, lastString);
	lastIndex=0;

	if (cmpStrings) {
	  tree = deleteString(tree, lastString, 0);
	} else {
	  tree2 = deleteString(tree2, NULL, lastNumber);
	}

	fputs(stringBuffer, fout);
	free(lastString);
  } else {
	fputs(NOT_FOUND, fout);
  }
}

void errorCommand() {
  fputs(SYNTAX_ERROR, fout);
}

void processCommand(char *buffer) {
  if (buffer[0] == 'A' && buffer[1] == 'D' && buffer[2] == 'D' && buffer[3] == ' ') {
	addCommand(buffer+4);
  } else {
	if (buffer[2] == ' ') {
	  switch (*buffer) {
		case 'F': {
		  switch (*(buffer+1)) {
			case 'S': {
			  fsCommand(buffer+3);
			  break;
			}
			case 'N': {
			  fnlCommand(buffer+3, tree2);
			  break;
			}
			case 'L': {
			  fnlCommand(buffer+3, tree);
			  break;
			}
			default: {
			  errorCommand();
			}
		  }
		  break;
		}
		case 'D': {
		  switch (*(buffer+1)) {
			case 'S': {
			  dsCommand(buffer+3);
			  break;
			}
			case 'N': {
			  dnlCommand(buffer+3, 0);
			  break;
			}
			case 'L': {
			  dnlCommand(buffer+3, 1);
			  break;
			}
			default: {
			  errorCommand();
			}
		  }
		  break;
		}
		default: {
		  errorCommand();
		}
	  }
	} else {
	  errorCommand();
	}
  }
}

int main() {
  fin = fopen("CATALOG.INP", "rt");
  fout = fopen("CATALOG.OUT", "wt");

  srand(3267);
  char buffer[301];

  while (fgets(buffer, 300, fin)) {
    if (buffer[0] == ' ') break;
   	processCommand(buffer);
  }

  fclose(fin);
  fclose(fout);
  return 0;
}