#ifndef CATALOG_H
#define CATALOG_H

#pragma warning(disable:4786)

#include <string>
//#include "qvector"//To do: New container type: "quick vector" with quick erase() and insert(), but a bit slower index operation.
#include <vector>
#include <map>
#include <list>

using namespace std;

typedef map<string,pair<int,int>,less<string> > STRINT;
//typedef vector<const STRINT::iterator> VECTSTR;//VC objects to this, unfortunately. So:
typedef vector<const string*> VECTSTR;
typedef list<pair<int,int> > PAIRLIST;
typedef list<int> INTLIST;//This may be vector as well

enum command{NOOP,ADD,DS,DN,DL,FS,FN,FL,ERR};
//Default minimal container size at which quick/dirty operations are permitted
int const MIN_DIRTY = 10000;
//Default maximal number of dirty (buffered) delete operations admitted:
int const MAX_DIRTY = 20000;
//Default maximal number of  dirty (buffered) Find operations admitted:
int const MAX_DIRTY_FIND = 50000;

class Catalog{
public: //Interface:

	Catalog():m_container(),m_ind(),m_indL(),m_DelBuffer(),m_DelBufferL(),
						m_DelCache(),m_DelCacheL(),
						m_dirty(false),toDel(0),dirtyFind(0),dirtyFindL(0),//currCommand(NOOP),
						minDirty(MIN_DIRTY),maxDirty(MAX_DIRTY),maxDirtyFind(MAX_DIRTY_FIND){};
	bool Add(const string, bool dirty=true);
	int Del(const string, bool dirty=true);
	string Del(int, bool dirty=true);
	string DelL(int, bool dirty=true);
	int Find(const string, bool dirty=true);
	string Find(int, bool dirty=true);
	string FindL(int, bool dirty=true);
	int size()const{return m_container.size()-toDel;}

	//command currCommand;
	int minDirty;//Minimal container size at which quick/dirty operations are permitted
	int maxDirty;//Maximal number of dirty (buffered) operations admitted
	int maxDirtyFind;//Maximal number of dirty (buffered) operations admitted

private: //Realization:

	//Auxiliary member functions:
	void MakeIndex();
	void MakeIndexL();
	void RebuildAll();

	int trueIndexW(int ind,PAIRLIST& DelBuffer);//Gets true index out of dirty one adding it to DelBuff
	int dirtyIndexW(int ind,PAIRLIST& DelBuffer);//Gets derty index out of true one adding it to DelBuff
	int trueIndex(int ind,const PAIRLIST& DelBuffer) const;//Read-only version of trueIndexW;
	int dirtyIndex(int ind,const PAIRLIST& DelBuffer) const;//Read-only version dirtyIndexW;

	void clearDelCache();
	void clearDelCacheL();

	int MaxDirtyFind(){
		//To Do:
		return maxDirtyFind;
	}

	//Data:
	STRINT m_container;//main container realized as map (balanced tree)
	VECTSTR m_ind;//index vector
	VECTSTR m_indL;//Lex index vector
	//Postpone erase operations until sufficient number of them
	//is accumulated in lists m_DelBuffer (for m_ind) or m_DelBufferL (for m_indL):
	PAIRLIST m_DelBuffer;//List of "index intervals" to be deleted from m_ind;
	PAIRLIST m_DelBufferL;//List of "index intervals" in m_indL to be deleted;
	//Delayed deletes are cached as well to the moment when the index info is really needed:
	INTLIST m_DelCache;//List of "dirty" indices to be deleted from m_ind;
	INTLIST m_DelCacheL;//List of "dirty" indices to be deleted from m_indL;

	bool m_dirty;//Dirty Lex index : if true the index vector m_indL must be rebuild before use
	int toDel;//Current number of "dirty" elements to be deleted from containers;
	int dirtyFind;
	int dirtyFindL;
};
#endif