#include <iostream>
#include <fstream>
#include <stdexcept>
#include "Catalog.h"
#include "miniSQL.h"
#include "timer.h"

int main(int argc, char **argv){
	Catalog cat;
	//Default optimization parameters:
	cat.maxDirty = 200;
	cat.minDirty = 1000;
	cat.maxDirtyFind = 500;
	//********************************
	//*   Parse Command Line String *
	//********************************
	for (int OptNum = 1; OptNum < argc; OptNum++)
	{
		string str = strupr(argv[OptNum]);
		string i_str = str.substr(1);
		int i = atoi(i_str.c_str());

		switch(str[0]){
	 		case 'M' :
	 			cat.minDirty = i;
	 			break;
	 		case 'D' :
	 			cat.maxDirty = i;
	 			break;
	 		case 'F' :
	 			cat.maxDirtyFind = i;
	 	}
	}
	ifstream is("Catalog.inp");
	if(!is){
		cout<<"Can not open Catalog.inp!"<<endl;
		exit(1);
	}
	ofstream os("Catalog.out");
	if(!os){
		cout<<"Can not open Catalog.out!"<<endl;
		exit(1);
	}
	int commd_count = 0, add_count = 0, del_count = 0, find_count = 0;
	timer time;
	time_t curr_time = 0;
	try{
	while(!is.eof()){
		string str;
		bool delCmd = false;//Is current command Delete?
		getline(is,str);
		switch(str[0]){
		case 'A' :
			++add_count;
			break;
		case 'D' :
			++del_count;
			delCmd = true;
			break;
		case 'F' :
			++find_count;
			//break;
		}
		str = execute(str,cat);
		os<<str;
		int ShowAll = 5000, ShowDel = 10;
		int ShowInterval = 2;//Show progress interval
		if(++commd_count%ShowAll==0){
			time.fix();
			if(time.total_sec-curr_time>=ShowInterval){
			curr_time = time.total_sec;
			cout<<commd_count<<" commands done (";
			if(add_count)cout<<"Add: "<<add_count<<" ";
			if(del_count)cout<<"Del: "<<del_count<<" ";
			if(find_count)cout<<"find: "<<find_count<<" ";
			time.print(cout);
			cout<<")..."<<endl;
			}
		}
		else
		if(delCmd&&del_count%ShowDel==0){
			time.fix();
			if(time.total_sec-curr_time>=ShowInterval){
			curr_time = time.total_sec;
			cout<<del_count<<" Del commands executed (";
			time.print(cout);
			cout<<")..."<<endl;
			}
		}
	}
	time.fix();
	cout<<commd_count<<" total commands executed (";
	if(add_count)cout<<"Add: "<<add_count<<" ";
	if(del_count)cout<<"Del: "<<del_count<<" ";
	if(find_count)cout<<"find: "<<find_count<<" ";
	time.print(cout);
	cout<<")."<<endl;
	is.close();
	os.close();
	}
	catch(out_of_range e){cout<<"Out of range bug: "<<e.what()<<"\n"; exit(1);}
	catch(invalid_argument e){cout<<"Invalid argument: "<<e.what()<<"\n"; exit(2);}
	catch(length_error e){cout<<"Length error: "<<e.what()<<"\n"; exit(4);}
	catch(range_error e){cout<<"Range error: "<<e.what()<<"\n"; exit(8);}
	catch(runtime_error e){cout<<"Run time error: "<<e.what()<<"\n"; exit(8);}
	catch(...){cout<<"Some dirty unknown bug happens!!! Sorry!\n"; exit(32);}
	return 0;
}